#!/usr/bin/env python
"""Unit tests for analysis.py: substitution matrix analysis code."""
from cogent.seqsim.analysis import tree_threeway_counts, \
    tree_twoway_counts, counts_to_probs, probs_to_rates, \
    tree_threeway_rates, tree_twoway_rates, \
    rates_to_array, multivariate_normal_prob
from cogent.seqsim.tree import RangeNode
from cogent.core.usage import DnaPairs, ABPairs
from cogent.seqsim.usage import Rates, Counts, Probs
from numpy import array, average, ones, zeros, float64, ravel, diag, any
from numpy.random import random, randint
from copy import deepcopy
from cogent.parse.tree import DndParser
from cogent.util.unit_test import TestCase, main

__author__ = "Rob Knight"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Rob Knight", "Daniel McDonald"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"

def occasionally_fails(test_method):
    # For tests that randomly fail occasionally, repeating them up to 5 times
    # and checking that one passes is fairly safe
    def wrapped(*args):
        for i in range(4):
            try:
                return test_method(*args)
            except AssertionError:
                pass
        return test_method(*args)
    wrapped.__doc__ = test_method.__doc__
    return wrapped

class analysisTests(TestCase):
    """Tests of top-level functions."""
    def setUp(self):
        """Make a couple of standard trees"""
        self.t1 = DndParser('((a,(b,c)),(d,e))', RangeNode)
        #selt.t1 indices: ((0,(1,2)5)6,(3,4)7)8
 
    def test_threeway_counts(self):
        """threeway_counts should produce correct count matrix"""
        self.t1.makeIdIndex()
        ind = self.t1.IdIndex
        ind[0].Sequence = array([0,0,0])
        ind[1].Sequence = array([0,1,0])
        ind[2].Sequence = array([1,0,1])
        ind[3].Sequence = array([1,1,0])
        ind[4].Sequence = array([1,1,1])
        depths = self.t1.leafLcaDepths()
        result = tree_threeway_counts(self.t1, depths, ABPairs)
        #check we got the right number of comparisons
        self.assertEqual(len(result), 20)
        #check we got the right keys
        for k in [(1,2,0),(2,1,0),(0,1,3),(1,0,3),(0,1,4),(1,0,4),(0,2,3),\
            (2,0,3),(0,2,4),(2,0,4),(1,2,3),(2,1,3),(1,2,4),(2,1,4),(3,4,1),\
            (4,3,1),(3,4,2),(4,3,2)]:
            assert k in result
        #spot-check a few results
        self.assertEqual(result[(1,2,0)]._data, array([[2,1],[0,0]]))
        self.assertEqual(result[(2,1,0)]._data, array([[1,2],[0,0]]))
        self.assertEqual(result[(2,1,3)]._data, array([[0,1],[1,1]]))
        
    def test_twoway_counts(self):
        """twoway_counts should produce correct count matrix"""
        self.t1.makeIdIndex()
        ind = self.t1.IdIndex
        ind[0].Sequence = array([0,0,0])
        ind[1].Sequence = array([0,1,0])
        ind[2].Sequence = array([1,0,1])
        ind[3].Sequence = array([1,1,0])
        ind[4].Sequence = array([1,1,1])
        depths = self.t1.leafLcaDepths()
        #check that it works with averaging
        result = tree_twoway_counts(self.t1, ABPairs)
        #check we got the right number of comparisons: average by default
        self.assertEqual(len(result), 10)
        #check we got the right keys
        for k in [(0,1),(0,2),(0,3),(0,4),(1,2),(1,3),(1,4),(2,3),(2,4),(3,4)]:
            assert k in result
        #spot-check a few results
        self.assertEqual(result[(0,1)]._data, array([[2,.5],[.5,0]]))
        self.assertEqual(result[(2,3)]._data, array([[0,1],[1,1]]))
        #check that it works when we don't average
        result = tree_twoway_counts(self.t1, ABPairs, average=False)
        self.assertEqual(len(result), 20)
        #check we got the right keys
        for k in [(0,1),(0,2),(0,3),(0,4),(1,2),(1,3),(1,4),(2,3),(2,4),(3,4)]:
            assert k in result
            #reverse should be in result too
            assert (k[1],k[0]) in result
        #spot-check values
        self.assertEqual(result[(0,1)]._data, array([[2,1],[0,0]]))
        self.assertEqual(result[(1,0)]._data, array([[2,0],[1,0]]))
        
    def test_counts_to_probs(self):
        """counts_to_probs should skip cases with zero rows"""
        counts = {
            (0,1): Counts(array([[0,1],[1,0]]), ABPairs),
            (1,2): Counts(array([[0,0],[1,0]]), ABPairs),           #bad row
            (0,3): Counts(array([[0,0],[0,0]]), ABPairs),           #bad row
            (0,4): Counts(array([[0.0,0.0],[0.0,0.0]]), ABPairs),   #bad row
            (0,5): Counts(array([[0.1,0.3],[0.0,0.0]]), ABPairs),   #bad row
            (3,4): Counts(array([[0.1,0.3],[0.4,0.1]]), ABPairs),
            (2,1): Counts(array([[0,5],[1,0]]), ABPairs),
            }
        result = counts_to_probs(counts)
        self.assertEqual(len(result), 3)
        self.assertFloatEqual(result[(0,1)]._data, array([[0,1],[1,0]]))
        self.assertFloatEqual(result[(3,4)]._data, \
            array([[0.25,0.75],[0.8,0.2]]))
        self.assertFloatEqual(result[(2,1)]._data, array([[0,1],[1,0]]))

    def test_probs_to_rates(self):
        """probs_to_rates converts probs to rates, omitting problem cases"""
        probs = dict([(i, Probs.random(DnaPairs)) for i in range(100)])
        rates = probs_to_rates(probs)
        #check we got at most the same number of items as in probs
        assert len(rates) <= len(probs)
        #check that we didn't get anything bad
        vals = rates.values()
        for v in vals:
            assert not v.isSignificantlyComplex()
        #check that we didn't miss anything good
        for key, val in probs.items():
            if key not in rates:
                try:
                    r = val.toRates()
                    print r.isValid()
                    assert r.isSignificantlyComplex() or (not r.isValid())
                except (ZeroDivisionError, OverflowError, ValueError):
                    pass

    def test_rates_to_array(self):
        """rates_to_array should pack rates into array correctly"""
        m1 = array([[-1,1,1,1],[2,-2,2,2],[3,3,-3,3],[1,2,3,-4]])
        m2 = m1 * 2
        m3 = m1 * 0.5
        m4 = zeros((4,4))
        m5 = array([0,0])
        r1, r2, r3, r4, r5 = [Rates(i, DnaPairs) for i in m1,m2,m3,m4,m5]
    
        data = {(0,1,0):r1, (1,2,0):r2, (2,0,0):r3, (2,1,1):r4}
        
        #note that array can be, but need not be, floating point
        to_fill = zeros((3,3,3,16), 'float64')
        result = rates_to_array(data, to_fill)
        #check that the thnigs we deliberately set are OK
        self.assertEqual(to_fill[0][1][0], ravel(m1))
        self.assertNotEqual(to_fill[0][1][0], ravel(m2))
        self.assertEqual(to_fill[1,2,0], ravel(m2))
        self.assertEqual(to_fill[2][0][0], ravel(m3))
        self.assertEqual(to_fill[2][1][1], ravel(m4))
        #check that everything else is zero
        nonzero = [(0,1,0),(1,2,0),(2,0,0)]
        for x in [(i, j, k) for i in range(3) for j in range(3) \
            for k in range(3)]:
            if x not in nonzero:
                self.assertEqual(to_fill[x], zeros(16))
        #check that it works omitting the diagonal
        to_fill = zeros((3,3,3,12), 'float64')
        result = rates_to_array(data, to_fill, without_diagonal=True)
        #check that the thnigs we deliberately set are OK
        m1_nodiag = array([[1,1,1],[2,2,2],[3,3,3],[1,2,3]])
        self.assertEqual(to_fill[0][1][0], ravel(m1_nodiag))
        self.assertNotEqual(to_fill[0][1][0], ravel(m1_nodiag*2))
        self.assertEqual(to_fill[1,2,0], ravel(m1_nodiag*2))
        self.assertEqual(to_fill[2][0][0], ravel(m1_nodiag*0.5))
        self.assertEqual(to_fill[2][1][1], zeros(12))
        #check that everything else is zero
        nonzero = [(0,1,0),(1,2,0),(2,0,0)]
        for x in [(i, j, k) for i in range(3) for j in range(3) \
            for k in range(3)]:
            if x not in nonzero:
                self.assertEqual(to_fill[x], zeros(12))
    
    @occasionally_fails
    def test_tree_threeway_rates(self):
        """tree_threeway_rates should give plausible results on rand trees"""
        t = self.t1
        t.assignLength(0.05)
        t.Q = Rates.random(DnaPairs).normalize()
        t.assignQ()
        t.assignP()
        t.evolve(randint(0,4,100))
        t.makeIdIndex()
        depths = t.leafLcaDepths()
        result = tree_threeway_rates(t, depths)
        self.assertEqual(result.shape, (5,5,5,16))
        #check that row sums are 0
        for x in [(i,j,k) for i in range(5) for j in range(5) \
            for k in range(5)]:
            self.assertFloatEqual(sum(result[x]), 0)
        assert any(result)
        #check that it works without_diag
        result = tree_threeway_rates(t, depths, without_diag=True)
        self.assertEqual(result.shape, (5,5,5,12))
        #check that it works with/without normalize
        #default: no normalization, so row sums shouldn't be 1 after 
        #omitting diagonal
        result = tree_threeway_rates(t, depths, without_diag=True)
        self.assertEqual(result.shape, (5,5,5,12))
        for x in [(i,j,k) for i in range(5) for j in range(5) \
            for k in range(5)]:
            assert sum(result[x]) == 0 or abs(sum(result[x]) - 1) > 0.01
        #...but if we tell it to normalize, row sums should be nearly 1
        #after omitting diagonal
        result = tree_threeway_rates(t, depths, without_diag=True, \
            normalize=True)
        self.assertEqual(result.shape, (5,5,5,12))
        for x in [(i,j,k) for i in range(5) for j in range(5) \
            for k in range(5)]:
                s = sum(result[x])
                if s != 0:
                    self.assertFloatEqual(s, 1)
    
    @occasionally_fails
    def test_tree_twoway_rates(self):
        """tree_twoway_rates should give plausible results on rand trees"""
        t = self.t1
        t.assignLength(0.05)
        t.Q = Rates.random(DnaPairs).normalize()
        t.assignQ()
        t.assignP()
        t.evolve(randint(0,4,100))
        t.makeIdIndex()
        result = tree_twoway_rates(t)
        self.assertEqual(result.shape, (5,5,16))
        #check that row sums are 0
        for x in [(i,j) for i in range(5) for j in range(5)]:
            self.assertFloatEqual(sum(result[x]), 0)
        #need to make sure we didn't just get an empty array
        self.assertGreaterThan((abs(result)).sum(), 0)
        #check that it works without_diag
        result = tree_twoway_rates(t, without_diag=True)
        self.assertEqual(result.shape, (5,5,12))
        #check that it works with/without normalize
        #default: no normalization, so row sums shouldn't be 1 after omitting
        #diagonal
        result = tree_twoway_rates(t, without_diag=True)
        self.assertEqual(result.shape, (5,5,12))
        #check that the row sums are not 1 before normalization (note that they
        #can be zero, though)
        sums_before = []
        for x in [(i,j) for i in range(5) for j in range(5)]:
            curr_sum = sum(result[x])
            sums_before.append(curr_sum)
        #...but if we tell it to normalize, row sums should be nearly 1
        #after omitting diagonal
        result = tree_twoway_rates(t, without_diag=True, \
            normalize=True)
        self.assertEqual(result.shape, (5,5,12))
        sums_after = []
        for x in [(i,j) for i in range(5) for j in range(5)]:
            curr_sum = sum(result[x])
            sums_after.append(curr_sum)
            if curr_sum != 0:
                self.assertFloatEqual(curr_sum, 1)
        try:
            self.assertFloatEqual(sums_before, sums_after)
        except AssertionError:
            pass
        else:
            raise AssertionError, "Expected different arrays before/after norm"
   
    def test_multivariate_normal_prob(self):
        """Multivariate normal prob should match R results"""
        cov = array([[3,1,2],[1,5,4],[2,4,6]])
        a = array([0,0,0])
        b = array([1,1,1])
        c = array([0.1, 0.2, 0.3])
        small_cov = cov/10.0
        
        mvp = multivariate_normal_prob
        self.assertFloatEqual(mvp(a, cov), 0.01122420)
        self.assertFloatEqual(mvp(a, cov, b), 0.009018894)
        self.assertFloatEqual(mvp(a, small_cov, b), 0.03982319)
        self.assertFloatEqual(mvp(c, small_cov, b), 0.06091317)
        
if __name__ == "__main__":
    main()




__author__ = "Julia Goodrich"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Jens Reeder","Julia Goodrich"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Julia Goodrich"
__email__ = "julia.goodrich@colorado.edu"
__Status__ = "Development"


from copy import copy
from types import GeneratorType

from numpy import transpose
from numpy.random import multinomial

from cogent.util.unit_test import FakeRandom
from cogent.core.sequence import Sequence
from cogent.parse.flowgram_parser import parse_sff
from cogent.parse.flowgram import Flowgram
from cogent.core.alignment import SequenceCollection

default_floworder = "TACG"
default_keyseq = "TCAG"

def assign_sequential_names(ignored, num_seqs, base_name='seq', start_at=0):
    """Returns list of num_seqs sequential, unique names.
    
    First argument is ignored; expect this to be set as a class attribute.
    """
    return ['%s_%s' % (base_name,i) for i in range(start_at,start_at+num_seqs)]

def flows_from_array(a):
    """FlowgramCollection from array of pos x seq: names are integers.
    
    This is an InputHandler for FlowgramCollection. It converts an arbitrary
    array of numbers into Flowgram objects and leaves the flowgrams unlabeled.
    """
    return list(transpose(a)), None, None

def flows_from_flowCollection(flow):
    """FlowgramCollection from array of pos x seq: names are integers.
    
    This is an InputHandler for FlowgramCollection. It converts an arbitrary
    array of numbers into Flowgram objects and leaves the flowgrams unlabeled.
    """
    return flow.flows, flow.Names, [f.header_info for f in flow.flows]

def flows_from_kv_pairs(flows):
    """SequenceCollection from list of (key, val) pairs.

        val can be str or flowgram object
    """
    
    names, flows =  map(list, zip(*flows))
    if isinstance(flows[0], str):
        info = [None]*len(flows)
    else:
        info = [f.header_info for f in flows]
    
    return flows, names, info

def flows_from_empty(obj, *args, **kwargs):
    """SequenceCollection from empty data: raise exception."""
    raise ValueError, "Cannot create empty SequenceCollection."

def flows_from_dict(flows):
    """SequenceCollection from dict of {label:flow_as_str} or {label:flow_obj}.
    """
    names, flows = map(list, zip(*flows.items()))
    if isinstance(flows[0], str):
        info = [None]*len(flows)
    else:
        info = [f.header_info for f in flows]
    return flows, names, info

def flows_from_sff(flows):
    """lines is sff file lines.
    """
    if isinstance(flows, str):
        flows = flows.splitlines()
        
    flows, head = parse_sff(flows)
    return flows_from_generic(flows)

def flows_from_generic(flows):
    """SequenceCollection from generic seq x pos data: seq of seqs of chars.
    
    This is an InputHandler for SequenceCollection. It converts a generic list
    (each item in the list will be mapped onto an object using
    seq_constructor and assigns sequential integers (0-based) as names.
    """
    names = []
    info = []
    for f in flows:
        if hasattr(f, 'Name'):
            names.append(f.Name)
        else:
            names.append(None)
        if hasattr(f, 'header_info'):
            info.append(f.header_info)
        else:
            info.append(None)
            
    return flows, names, info


class FlowgramCollection(object):
    """stores Flowgrams.
    
    - InputHandlers: support for different data types
    - flows: behaves like list of Flowgram objects
    - Names: behaves like list of names for the Flowgram objects
    - NamedFlows: behaves like dict of {name:flow}
    """    
    InputHandlers = {   'array': flows_from_array,
                        'dict': flows_from_dict,
                        'sff' :flows_from_sff,
                        'empty': flows_from_empty,
                        'kv_pairs':flows_from_kv_pairs,
                        'flowcoll':flows_from_flowCollection,
                        'generic':flows_from_generic
                    }
    HeaderInfo = ['Magic Number', 'Version', 'Index Offset', 'Index Length',
                  '# of Reads', 'Header Length', 'Key Length', '# of Flows',
                  'Flowgram Code', 'Flow Chars', 'Key Sequence']

    DefaultNameFunction = assign_sequential_names
    
    def __init__(self, data, Name = None, Names = None,
                 header_info = None, conversion_f = None,
                 name_conversion_f = None, remove_duplicate_names = False):
        """Initialize self with data and optionally Info.
        

        Parameters:
        
        data:           Data to convert into a FlowgramCollection
        
        Name:           Name of the FlowgramCollection.
                
        conversion_f:   Function to convert data into Flowgram.
                        
        name_conversion_f: if present, converts name into f(name).

        header_info: contains info to be printed in the common header of an sff
                    file, it is a dictionary. ex: Key Sequence:"ATCG"
        """

        #read all the data in if we were passed a generator
        if isinstance(data, GeneratorType):
            data = list(data)

        #set the Name
        self.Name = Name

        if header_info is not None:
            self._check_header_info(header_info)
        
            for i in header_info:
                setattr(self,i,header_info[i])
            if 'Key Sequence' in header_info:
                keyseq = header_info['Key Sequence']
            else:
                keyseq = None
            if 'Flow Chars' in header_info:
                floworder = header_info['Flow Chars']
            else:
                floworder = default_floworder
        else:
            keyseq = None
            floworder = default_floworder


        self.header_info = header_info

        per_flow_names, flows, name_order, info = \
                self._names_flows_order(conversion_f, data, Names, \
                name_conversion_f, remove_duplicate_names)
        self.Names = name_order

        #will take only the flows and names that are in name_order
        if per_flow_names != name_order:
            good_indices = []
            for n in name_order:
                good_indices.append(per_flow_names.index(n))
            flows = [flows[i] for i in good_indices]
            info = [info[i] for i in good_indices]
            per_flow_names = name_order

        self.flow_str= flows  
        self.flows = [Flowgram(f,n,keyseq,floworder, i)\
                      for f,n, i in zip(flows,self.Names,info)]

        #create NamedFlows dict for fast lookups
        self.NamedFlows = self._make_named_flows(self.Names, self.flows)
                

    def _strip_duplicates(self, names, flows, info):
        """Internal function to strip duplicates from list of names"""
        if len(set(names)) == len(names):
            return set(), names, flows, info
        #if we got here, there are duplicates
        unique_names = {}
        duplicates = {}
        fixed_names = []
        fixed_flows = []
        fixed_info = []
        for n, f, i in zip(names, flows,info):
            if n in unique_names:
                duplicates[n] = 1
            else:
                unique_names[n] = 1
                fixed_names.append(n)
                fixed_flows.append(f)
                fixed_info.append(i)
                
        return duplicates, fixed_names, fixed_flows, fixed_info


    def _names_flows_order(self, conversion_f, data, Names, \
            name_conversion_f, remove_duplicate_names):
        """Internal function to figure out names, flows, and name_order."""
        #figure out conversion function and whether it's an array
        if not conversion_f:
            input_type = self._guess_input_type(data)
            conversion_f = self.InputHandlers[input_type]
        #set seqs, names, and handler_info as properties
        
        flows, names, info = conversion_f(data)
        if names and name_conversion_f:
            names = map(name_conversion_f, names)

        #if no names were passed in as Names, if we obtained them from
        #the seqs we should use them, but otherwise we should use the
        #default names
        if Names is None:
            if (names is None) or (None in names):
                per_flow_names = name_order = \
                    self.DefaultNameFunction(len(flows))
            else:   #got names from seqs
                per_flow_names = name_order = names
        else:
            #otherwise, names were passed in as Names: use this as the order
            #if we got names from the sequences, but otherwise assign the
            #names to successive sequences in order
            if (names is None) or (None in names):
                per_flow_names = name_order = Names
            else: #got names from seqs, so assume name_order is in Names
                per_flow_names = names
                name_order = Names
        #check for duplicate names
        duplicates, fixed_names, fixed_flows, fixed_info = \
            self._strip_duplicates(per_flow_names, flows, info)
        if duplicates:
            if remove_duplicate_names:
                per_flow_names, flows, info =fixed_names,fixed_flows,fixed_info
                #if name_order doesn't have the same names as per_seq_names,
                #replace it with per_seq_names
                if (set(name_order) != set(per_flow_names)) or\
                    (len(name_order) != len(per_flow_names)):
                    name_order = per_flow_names
            else:
                raise ValueError, \
                "Some names were not unique. Duplicates are:\n" + \
                str(sorted(duplicates.keys()))
        return per_flow_names, flows, name_order, info


    def _check_header_info(self, info):
        for h in info:
            if h not in self.HeaderInfo:
                raise ValueError, "invalid key in header_info"
        
    def _make_named_flows(self, names, flows):
        """Returns NamedFlows: dict of name:flow."""
        name_flow_tuples = zip(names, flows)
        for n, f in name_flow_tuples:
            f.Name = n
        return dict(name_flow_tuples)
    
    def _guess_input_type(self, data):
        """Guesses input type of data; returns result as key of InputHandlers.
               
        Returns 'empty' if check fails, i.e. if it can't recognize the data
        as a specific type. Note that bad data is not guaranteed to
        return 'empty', and may be recognized as another type incorrectly.
        """
        
        if isinstance(data, dict):
            return 'dict'
        if isinstance(data, str):
            return 'sff'
        if isinstance(data, FlowgramCollection):
            return 'flowcoll'
        
        first = None
        try:
            first = data[0]
        except (IndexError, TypeError):
            pass
        try:
            first = iter(data).next()
        except (IndexError, TypeError, StopIteration):
            pass
        if first is None:
            return 'empty'
        try:
            if isinstance(first, Flowgram):     #model sequence base type
                return 'generic'
            if hasattr(first, 'dtype'):    #array object
                return 'array'
            elif isinstance(first, str) and first.startswith('Common'):
                return 'sff'
            else:
                try:
                    dict(data)
                    return 'kv_pairs'
                except (TypeError, ValueError):
                    pass
            return "generic"
        except (IndexError, TypeError), e:
            return 'empty'

    def __len__(self):
        """returns the number of flowgrams in the collection"""
        return len(self.flows)

    def __iter__(self):
        """iterates over the flows in the collection"""
        for f in self.flows:
            yield f

    def __str__(self):
        """returns string like sff file given the flowgrams and header_info"""
        lines = self.createCommonHeader()
        lines.append('')
        lines.extend([f.createFlowHeader() for f in self.flows])
        return '\n'.join(lines)

    def __cmp__(self, other):
        """cmp first tests as dict, then as str."""
        c = cmp(self.NamedFlows, other)
        if not c:
            return 0
        else:
            return cmp(str(self), str(other))

    def keys(self):
        """keys uses self.Names
        
        Note: returns copy, not original.
        """
        return self.Names[:]
    
    def values(self):
        """values returns values corresponding to self.Names."""
        return [self.NamedFlows[n] for n in self.Names]

    def items(self):
        """items returns (name, value) pairs."""
        return [(n, self.NamedFlows[n]) for n in self.Names]
    
    def writeToFile(self, filename=None, **kwargs):
        """Write the flowgrams to a file.
        
        Arguments:
        - filename: name of the output file
        
        """
        
        if filename is None:
            raise DataError('no filename specified')
        
        f = open(filename, 'w')
        f.write(str(self))
        f.close()

    def createCommonHeader(self):
        """header_info dict turned into flowgram common header"""
        lines = ["Common Header:"]
        
        if self.header_info is not None:
            lines.extend(["  %s:\t%s" % (param,self.header_info[param]) \
                      for param in self.header_info])

        return lines

    def toFasta(self, exclude_ambiguous = False,Bases=False,
                make_seqlabel = None):
        """Return flowgram collection in Fasta format
        
        Arguments:
            - make_seqlabel: callback function that takes the seq object and
              returns a label str

        if Bases is True then a fasta string will be made using
            self.Bases instead of translating the flowgram
              
        """
        if exclude_ambiguous:
            flows = flows.omitAmbiguousFlows()
        else:
            flows = self

        seqs = flows.toSequenceCollection(Bases)
            
        return seqs.toFasta(make_seqlabel = make_seqlabel)

    def toPhylip(self, exclude_ambiguous = False,Bases=False,generic_label=True,
                 make_seqlabel=None):
        """
        Return alignment in PHYLIP format and mapping to sequence ids
        
        raises exception if invalid alignment
        
        Arguments:
            - make_seqlabel: callback function that takes the seq object and
              returns a label str

        if Bases is True then a fasta string will be made using
            self.Bases instead of translating the flowgram 
        """
        if exclude_ambiguous:
            flows = flows.omitAmbiguousFlows()
        else:
            flows = self

        seqs = flows.toSequenceCollection( Bases)
        return seqs.toPhylip(make_seqlabel = make_seqlabel,
                             generic_label= generic_label)
    
    def toNexus(self,seq_type, exclude_ambiguous = False, Bases = False,
                interleave_len=50):
        """
        Return alignment in NEXUS format and mapping to sequence ids
        
        **NOTE** Not that every sequence in the alignment MUST come from
            a different species!! (You can concatenate multiple sequences from
            same species together before building tree)
        
        seq_type: dna, rna, or protein
        
        Raises exception if invalid alignment
        if Bases is True then a fasta string will be made using
        self.Bases instead of translating the flowgram
        """

        if exclude_ambiguous:
            flows = flows.omitAmbiguousFlows()
        else:
            flows = self

        seqs = flows.toSequenceCollection(Bases)
        return seqs.toNexus(seq_type,interleave_len = interleave_len)
    
    def toSequenceCollection(self, Bases = False):
        names = self.Names
        flow_dict = self.NamedFlows
        flows = [flow_dict[f].toSeq(Bases = Bases) for f in names]
        return SequenceCollection(flows)

    def addFlows(self, other):
        """Adds flowgrams from other to self. Returns a new object.
        
        other must be of same class as self or coerceable to that class..
        """
        assert not isinstance(other, str), "Must provide a series of flows "+\
                                            "or an flowgramCollection"
        self_flow_class = self.flows[0].__class__
        try:
            combined = self.flows + other.flows
        except AttributeError:
            combined = self.flows + list(other)
        try:
            combined_info =copy(self.header_info)
            combined_info.update(other.header_info)
        except AttributeError:
            combined_info =self.header_info
        
        
        for flow in combined:
            assert flow.__class__ == self_flow_class,\
                "classes different: Expected %s, Got %s" % \
                    (flow.__class__, self_flow_class)
        return self.__class__(data=combined,header_info=combined_info)

    def iterFlows(self, flow_order=None):
        """Iterates over values (sequences) in the alignment, in order.
        
        seq_order: list of keys giving the order in which seqs will be returned.
        Defaults to self.Names. Note that only these sequences will be
        returned, and that KeyError will be raised if there are sequences
        in order that have been deleted from the Alignment. If self.Names
        is None, returns the sequences in the same order as
        self.NamedSeqs.values().
        
        Use map(f, self.seqs()) to apply the constructor f to each seq. f must
        accept a single list as an argument.
        
        Always returns references to the same objects that are values of the
        alignment.
        """
        ns = self.NamedFlows
        get = ns.__getitem__
        for key  in flow_order or self.Names:
            yield get(key)


    def iterItems(self, flow_order=None, pos_order=None):
        """Iterates over elements in the flowgram collection.
        
        seq_order (names) can be used to select a subset of seqs.
        pos_order (positions) can be used to select a subset of positions.
        
        Always iterates along a seq first, then down a position (transposes
        normal order of a[i][j]; possibly, this should change)..
        
        WARNING: FlowgramCollection.iterItems() is not the same as fc.iteritems()
        (which is the built-in dict iteritems that iterates over key-value
        pairs).
        """
        if pos_order:
            for row in self.iterFlows(flow_order):
                for i in pos_order:
                    yield row[i]
        else:
            for row in self.iterFlows(flow_order):
                for i in row:
                    yield i

    Items = property(iterItems)
                    
    def copy(self):
        """Returns deep copy of self."""
        result = self.__class__(self)

    def _take_flows(self): return list(self.iterFlows())
    
    Flows = property(_take_flows)   #access as attribute if using default order.
    
    def takeFlows(self, flows, negate=False, **kwargs):
        """Returns new FlowgramCollection containing only specified flows.
        
        Note that the flows in the new collection will be references to the
        same objects as the seqs in the old collection.
        """
        get = self.NamedFlows.__getitem__
        result = {}
        if negate:
            #copy everything except the specified seqs
            negated_names = []
            row_lookup = dict.fromkeys(flows)
            for r, row in self.NamedFlows.items():
                if r not in row_lookup:
                    result[r] = row
                    negated_names.append(r)
            flows = negated_names #remember to invert the list of names
        
        else:
            #copy only the specified seqs
            for r in flows:
                result[r] = get(r)
        if result:
            return self.__class__(result, Names=flows, **kwargs)
        else:
            return {}   #safe value; can't construct empty collection

    def getFlowIndices(self, f, negate=False, Bases = False):
        """Returns list of keys of flows where f(Seq) is True.
        
        List will be in the same order as self.Names, if present.
        If Bases is True it will use the flowgram Bases attribute otherwise
        it uses the translation to a sequence
        """
        get = self.NamedFlows.__getitem__
        #negate function if necessary
        if negate:
            new_f = lambda x: not f(x)
        else:
            new_f = f
        #get all the seqs where the function is True
        return [key for key in self.Names \
               if new_f(get(key).toSeq(Bases = Bases))]
    
    def takeFlowsIf(self, f, negate=False, Bases = False, **kwargs):
        """Returns new Alignment containing seqs where f(row) is True.
        
        Note that the seqs in the new Alignment are the same objects as the
        seqs in the old Alignment, not copies.

        If Bases is True it will use the flowgram Bases attribute otherwise
        it uses the translation to a sequence
        """
        #pass negate to get SeqIndices
        return self.takeFlows(self.getFlowIndices(f, negate, Bases= Bases),
                              **kwargs)

    def getFlow(self, flowname):
        """Return a flowgram object for the specified seqname.
        """
        return self.NamedFlows[flowname]

    def getFlowNames(self):
        """Return a list of Flowgram names."""
        return self.Names[:]

    def getIntMap(self,prefix='seq_'):
        """Returns a dict with names mapped to enumerates integer names.
            
            - prefix: prefix for sequence label. Default = 'seq_'
            - int_keys is a dict mapping int names to sorted original names.
        """
        get = self.NamedFlows.__getitem__
        int_keys = dict([(prefix+str(i),k) for i,k in \
                enumerate(sorted(self.NamedFlows.keys()))])
        int_map = dict([(k, copy(get(v))) for k,v in int_keys.items()])
        return int_map, int_keys

    def toDict(self):
        """Returns the collection as dict of names -> strings.

        Note: returns strings, NOT Flowgram objects.
        """
        collection_dict = {}
        
        for flow_name in self.Names:
            collection_dict[flow_name] = self.NamedFlows[flow_name]._flowgram
        
        return collection_dict
    
    def omitAmbiguousFlows(self, Bases = False):
        """Returns an object containing only the sequences without N's"""
        is_ambiguous = lambda x: 'N' not in x
        
        return self.takeFlowsIf(is_ambiguous, Bases = Bases)

    def setBases(self):
        """Sets the Bases property for each flowgram using toSeq"""
        for f in self.values():
            f.Bases = f.toSeq()
    

def pick_from_prob_density(pvals,bin_size):
    l = multinomial(1,pvals)
    return (l.nonzero()[0][0]) * bin_size

def seqs_to_flows(seqs, keyseq = default_keyseq, floworder = default_floworder,
                  numflows = None, probs = None, bin_size = 0.01,
                  header_info = {}):
    """ Transfrom a sequence into an ideal flow
        seqs: a list of name sequence object tuples (name,tuple)
        keyseq: the flowgram key Sequence
        floworder: The chars needed to convert seq to flow
        numflows: number of total flows in each flowgram, if it is specified
            the flowgram will be padded to that number
        probs: dictionary defining the probability distribution for each
            homopolymer

        WARNING:each distributions probabilities must add to 1.0
    """
    flows = []
    homopolymer_counter = 1.0
    if probs:
        for p in probs:
            if round(sum(probs[p]),1) != 1.0:
                raise ValueError, 'probs[%s] does not add to 1.0' % p

    for name,seq in seqs:
        flow_seq = FakeRandom(floworder,True)
        flow = []
        seq_len = len(seq)
        for i, nuc in enumerate(seq):
            if i < seq_len-1 and seq[i+1] == nuc:
                homopolymer_counter += 1.0
            else:
                while flow_seq() != nuc:
                    if probs is None:
                        val = 0.0
                    else:
                        val = pick_from_prob_density(probs[0],bin_size)
                    flow.append(val)
                if (probs is None) or (homopolymer_counter > 9):
                    val = homopolymer_counter
                else:
                    val = pick_from_prob_density(probs[int(homopolymer_counter)],bin_size)
                 
                flow.append(val)
                homopolymer_counter = 1.0

        len_flow = len(flow)
        len_order = len(floworder)

        if numflows is not None and numflows % len_order != 0:
            raise ValueError, "numflows must be divisable by the length of floworder"
        if (len_flow % len_order != 0):
            right_missing = len_order - (len_flow % len_order)
            if numflows != (len_flow + right_missing) and numflows is not None:
                right_missing += (numflows - (len_flow+right_missing))
            if probs is None:
                flow.extend([0.0]*right_missing)
            else:
                for i in range(0,right_missing):
                    flow.append(pick_from_prob_density(probs[0],bin_size))

        flows.append((name, Flowgram(flow, id, keyseq, floworder)))
    if keyseq is not None:
        keylen = len(keyseq)
    else:
        keylen = None
    header_info.update({'Key Sequence':keyseq,'Flow Chars':floworder,
                        'Key Length':keylen})
    return FlowgramCollection(flows, header_info = header_info)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
reload(sys)
sys.setdefaultencoding("UTF-8")

import doctest, os, sys, glob
import click

"""
This will doctest all files ending with .rst in this directory.
"""

@click.command()
@click.option('-f', '--file_paths', help="specific rst files to test")
@click.option('-j', '--just', help='comma separated list of names to be matched to files to be tested')
@click.option('-x', '--exclude', help='comma separated list of names to be matched to files to be excluded')
@click.option('-v', '--verbose', is_flag=True, help='verbose output')
def main(file_paths, just, exclude, verbose):
    """runs doctests for the indicated files"""
    if not file_paths:
        # find all files that end with rest
        file_paths = [fname for fname in os.listdir(os.getcwd()) if fname.endswith('.rst')]
    elif "*" in file_paths:
        file_paths = glob.glob(file_paths)
    
    if verbose:
        print file_paths
    
    if just:
        just = just.split(',')
        new = []
        for fn in file_paths:
            for sub_word in just:
                if sub_word in fn:
                    new.append(fn)
        file_paths = new
    elif exclude:
        exclude = exclude.split(',')
        new = []
        for fn in file_paths:
            keep = True
            for sub_word in exclude:
                if sub_word in fn:
                    keep = False
                    break
            if keep:
                new.append(fn)
        file_paths = new
    
    if verbose:
        print "File paths, after filtering: %s" % str(file_paths)
    
    for test in file_paths:
        print
        print "#" * 40
        print test
        doctest.testfile(test, optionflags=doctest.ELLIPSIS, verbose=True, encoding='utf-8')
    

if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""Parser for the default bowtie output
   Compatible with version 0.12.5
"""
from cogent import LoadTable
from cogent.parse.table import ConvertFields

__author__ = "Gavin Huttley, Anuj Pahwa"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Rob Knight","Peter Maxwell", "Gavin Huttley", "Anuj Pahwa"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Development"

# The 4th and the 7th elements of the row of data returned from bowtie are
# integer values and can thus be converted.
row_converter = ConvertFields([(3, int), (6, int)])

def BowtieOutputParser(data, row_converter=row_converter):
    """yields a header and row of data from the default bowtie output
    
    Arguments:
        - row_converter: if not provided, uses a default converter which casts
          the Offset and Other Matches fields to ints. If set to None, all
          returned data will be strings (this is faster).
    """
    
    header = ['Query Name', 'Strand Direction','Reference Name', 'Offset',
              'Query Seq', 'Quality', 'Other Matches', 'Mismatches']
    yield header
    
    # If given a filename for the data
    if type(data) == str:
        data = open(data)
    
    for record in data:
        row = record.rstrip('\n').split('\t')
        if row_converter:
            row = row_converter(row)
        
        # convert the last element to a list of strings
        if row[-1] is '':
            row[-1] = []
        else:
            row[-1] = row[-1].split(',')
        
        yield row
    

def BowtieToTable(data, row_converter=row_converter):
    """Converts bowtie output to a table
    
    Arguments:
        - row_converter: if not provided, uses a default converter which casts
          the Offset and Other Matches fields to ints. If set to None, all
          returned data will be strings (this is faster).
    """
    parser = BowtieOutputParser(data, row_converter=row_converter)
    header = parser.next()
    rows = [row for row in parser]
    table = LoadTable(header=header, rows=rows)
    return table



#!/usr/bin/env python

import os
import numpy as np
from numpy import sum
from cogent.util.unit_test import TestCase, main
from cogent.app.util import ApplicationNotFoundError
from cogent.parse.pdb import PDBParser
from cogent.struct.selection import einput
from cogent.maths.stats.test import correlation


__author__ = "Marcin Cieslik"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__contributors__ = ["Marcin Cieslik"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Marcin Cieslik"
__email__ = "mpc4p@virginia.edu"
__status__ = "Development"


class DummyFile(object):
    def __init__(self, some_string):
        self.some_string = some_string
        slist = self.some_string.split('\n')
        self.some_string_list = [i + '\n' for i in slist]
    def readlines(self):
        return self.some_string_list
    def close(self):
        pass


test_file_water = """
HETATM 1185  O   HOH   268     141.577  14.676  13.168  1.00 54.76           O
HETATM 1186  O   HOH   269     137.019  17.606  19.854  1.00 33.36           O
HETATM 1187  O   HOH   270     149.639  55.203   4.611  1.00 49.01           O
HETATM 1188  O   HOH   271     156.238  32.191  -4.204  1.00 64.53           O
CONECT  453  685
CONECT  685  453
MASTER      357    0    0    1   10    0    0    6 1187    1    2   12
END
"""

dummy_water = DummyFile(test_file_water)


class asaTest(TestCase):
    """Tests for surface calculations."""

    def setUp(self):
        self.arr = np.random.random(3000).reshape((1000, 3))
        self.point = np.random.random(3)
        self.center = np.array([0.5, 0.5, 0.5])

    def test_0import(self):
        # sort by name
        """tests if can import _asa cython extension."""
        global _asa
        from cogent.struct import _asa
        assert 'asa_loop' in dir(_asa)

    def test_1import(self):
        # sort by name
        """tests if can import asa."""
        global asa
        from cogent.struct import asa

    def test_asa_loop(self):
        """tests if inner asa_loop (cython) performs correctly"""
        self.lcoords = np.array([[-4., 0, 0], [0, 0, 0], [4, 0, 0], [10, 0, 0]])
        self.qcoords = np.array([[0., 0, 0], [4., 0, 0]])
        self.lradii = np.array([2., 3.])
        self.qradii = np.array([3., 2.])
        #spoints, np.ndarray[DTYPE_t, ndim =1] box,\
        # DTYPE_t probe, unsigned int bucket_size, MAXSYM =200000)
        self.spoints = np.array([[1., 0., 0.], [-1., 0., 0.], [0., 1., 0.], \
                                 [0., -1., 0.], [0., 0., 1.], [0., 0., -1.]])
        output = _asa.asa_loop(self.qcoords, self.lcoords, self.qradii, \
                               self.lradii, self.spoints, \
                    np.array([-100., -100., -100., 100., 100., 100.]), 1., 10)
        self.assertFloatEqual(output, np.array([ 75.39822369, 41.88790205]))

    def test_asa_xtra(self):
        """test internal asa"""
        self.input_file = os.path.join('data', '2E12.pdb')
        self.input_structure = PDBParser(open(self.input_file))
        self.assertRaises(ValueError, asa.asa_xtra, self.input_structure, mode='a')
        result = asa.asa_xtra(self.input_structure)
        a = einput(self.input_structure, 'A')
        for i in range(len(result)):
            self.assertEquals(result.values()[i]['ASA'], a[result.keys()[i]].xtra['ASA'])
        r = einput(self.input_structure, 'R')
        for water in  r.selectChildren('H_HOH', 'eq', 'name').values():
            self.assertFalse('ASA' in water.xtra)
        for residue in  r.selectChildren('H_HOH', 'ne', 'name').values():
            for a in residue:
                self.assertTrue('ASA' in a.xtra)
        result = asa.asa_xtra(self.input_structure, xtra_key='SASA')
        for residue in  r.selectChildren('H_HOH', 'ne', 'name').values():
            for a in residue:
                a.xtra['ASA'] == a.xtra['SASA']

    def test_asa_xtra_stride(self):
        """test asa via stride"""
        self.input_file = os.path.join('data', '2E12.pdb')
        self.input_structure = PDBParser(open(self.input_file))
        try:
            result = asa.asa_xtra(self.input_structure, 'stride')
        except ApplicationNotFoundError: 
            return
        self.assertAlmostEqual(self.input_structure[(0,)][('B',)]\
                               [(('LEU', 35, ' '),)].xtra['STRIDE_ASA'], 17.20)

    def test_compare(self):
        """compares internal asa to stride."""
        self.input_file = os.path.join('data', '2E12.pdb')
        self.input_structure = PDBParser(open(self.input_file))
        try:
            asa.asa_xtra(self.input_structure, mode='stride')
        except ApplicationNotFoundError: 
            return            
        asa.asa_xtra(self.input_structure)
        self.input_structure.propagateData(sum, 'A', 'ASA', xtra=True)
        residues = einput(self.input_structure, 'R')
        asa1 = []
        asa2 = []
        for residue in  residues.selectChildren('H_HOH', 'ne', 'name').values():
            asa1.append(residue.xtra['ASA'])
            asa2.append(residue.xtra['STRIDE_ASA'])
        self.assertAlmostEqual(correlation(asa1, asa2)[1], 0.)

    def test_uc(self):
        """compares asa within unit cell."""
        self.input_file = os.path.join('data', '2E12.pdb')
        self.input_structure = PDBParser(open(self.input_file))
        asa.asa_xtra(self.input_structure, symmetry_mode='uc', xtra_key='ASA_UC')
        asa.asa_xtra(self.input_structure)
        self.input_structure.propagateData(sum, 'A', 'ASA', xtra=True)
        self.input_structure.propagateData(sum, 'A', 'ASA_UC', xtra=True)
        residues = einput(self.input_structure, 'R')
        x = residues[('2E12', 0, 'B', ('GLU', 77, ' '))].xtra.values()
        self.assertTrue(x[0] != x[1])

    def test_uc2(self):
        self.input_file = os.path.join('data', '1LJO.pdb')
        self.input_structure = PDBParser(open(self.input_file))
        asa.asa_xtra(self.input_structure, symmetry_mode='uc', xtra_key='ASA_XTAL')
        asa.asa_xtra(self.input_structure)
        self.input_structure.propagateData(sum, 'A', 'ASA', xtra=True)
        self.input_structure.propagateData(sum, 'A', 'ASA_XTAL', xtra=True)
        residues = einput(self.input_structure, 'R')
        r1 = residues[('1LJO', 0, 'A', ('ARG', 65, ' '))]
        r2 = residues[('1LJO', 0, 'A', ('ASN', 46, ' '))]
        self.assertFloatEqual(r1.xtra.values(),
                              [128.94081270529105, 22.807700865674093])
        self.assertFloatEqual(r2.xtra.values(),
                              [115.35738419425566, 115.35738419425566])

    def test_crystal(self):
        """compares asa within unit cell."""
        self.input_file = os.path.join('data', '2E12.pdb')
        self.input_structure = PDBParser(open(self.input_file))
        asa.asa_xtra(self.input_structure, symmetry_mode='uc', crystal_mode=2, xtra_key='ASA_XTAL')
        asa.asa_xtra(self.input_structure)
        self.input_structure.propagateData(sum, 'A', 'ASA', xtra=True)
        self.input_structure.propagateData(sum, 'A', 'ASA_XTAL', xtra=True)
        residues = einput(self.input_structure, 'R')
        r1 = residues[('2E12', 0, 'A', ('ALA', 42, ' '))]
        r2 = residues[('2E12', 0, 'A', ('VAL', 8, ' '))]
        r3 = residues[('2E12', 0, 'A', ('LEU', 25, ' '))]
        self.assertFloatEqual(r1.xtra.values(), \
                                [32.041070749038823, 32.041070749038823])
        self.assertFloatEqual(r3.xtra.values(), \
                               [0., 0.])
        self.assertFloatEqual(r2.xtra.values(), \
                                [28.873559956056916, 0.0])

    def test__prepare_entities(self):
        self.input_structure = PDBParser(dummy_water)
        self.assertRaises(ValueError, asa._prepare_entities, self.input_structure)

    def _test_bio(self):
        """compares asa within a bio unit."""
        self.input_file = os.path.join('data', '1A1X.pdb')
        self.input_structure = PDBParser(open(self.input_file))
        asa.asa_xtra(self.input_structure, symmetry_mode='bio', xtra_key='ASA_BIO')
        asa.asa_xtra(self.input_structure)
        self.input_structure.propagateData(sum, 'A', 'ASA', xtra=True)
        self.input_structure.propagateData(sum, 'A', 'ASA_BIO', xtra=True)
        residues = einput(self.input_structure, 'R')
        r1 = residues[('1A1X', 0, 'A', ('GLU', 37, ' '))]
        r2 = residues[('1A1X', 0, 'A', ('TRP', 15, ' '))]
        self.assertFloatEqual(r1.xtra.values(), \
                                [20.583191467544726, 78.996394472066541])
        self.assertFloatEqual(r2.xtra.values(), \
                                [136.41436710386989, 136.41436710386989])





if __name__ == '__main__':
    main()



#!/usr/bin/env python

from align import make_dna_scoring_dict, _align_pairwise, \
                  classic_align_pairwise, local_pairwise, global_pairwise

__all__ = ['algorithm', 'align', 'dp_calculation', 'indel_model',
           'indel_positions', 'pairwise', 'partial_order_graph', 'progressive',
           'pycompare', 'traceback']

__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Peter Maxwell", "Jeremy Widmann", "Gavin Huttley",
                    "Rob Knight"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Production"



#!/usr/bin/env python
from setuptools import setup, Command
from setuptools.extension import Extension
import sys, os, re, subprocess

__author__ = "Peter Maxwell"
__copyright__ = "Copyright 2007-2011, The Cogent Project"
__contributors__ = ["Peter Maxwell", "Gavin Huttley", "Matthew Wakefield",
                    "Greg Caporaso", "Daniel McDonald"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Peter Maxwell"
__email__ = "pm67nz@gmail.com"
__status__ = "Production"

# Check Python version, no point installing if unsupported version inplace
if sys.version_info < (2, 6):
    py_version = ".".join([str(n) for n in sys.version_info])
    raise RuntimeError("Python-2.6 or greater is required, Python-%s used." % py_version)


# Check Numpy version, no point installing if unsupported version inplace
try:
    import numpy
except ImportError:
    raise RuntimeError("Numpy required but not found.")

numpy_version = re.split("[^\d]", numpy.__version__)
numpy_version_info = tuple([int(i) for i in numpy_version if i.isdigit()])
if numpy_version_info < (1, 3):
    raise RuntimeError("Numpy-1.3 is required, %s found." % numpy_version)

# Find arrayobject.h on any system
numpy_include_dir = numpy.get_include()


# On windows with no commandline probably means we want to build an installer.
if sys.platform == "win32" and len(sys.argv) < 2:
    sys.argv[1:] = ["bdist_wininst"]


# A new command for predist, ie: pyrexc but no compile.
class NullCommand(Command):
    description = "Generate .c files from .pyx files"
    # List of option tuples: long name, short name (or None), and help string.
    user_options = [] #[('', '', ""),]
    def initialize_options (self):
        pass
    def finalize_options (self):
        pass
    def run (self):
        pass
        
class BuildDocumentation(NullCommand):
    description = "Generate HTML documentation and .c files"
    def run (self):
        # Restructured Text -> HTML
        try:
            import sphinx
        except ImportError:
            print "Failed to build html due to ImportErrors for sphinx"
            return
        cwd = os.getcwd()
        os.chdir('doc')
        subprocess.call(["make", "html"])
        os.chdir(cwd)
        print "Built index.html"

# Cython is now run via the Cythonize function rather than monkeypatched into 
# distutils, so these legacy commands don't need to do anything extra.
extra_commands = {
    'pyrexc': NullCommand,
    'cython': NullCommand,
    'predist': BuildDocumentation}


# Compiling Pyrex modules to .c and .so, if possible and necessary
try:
    if 'DONT_USE_PYREX' in os.environ:
        raise ImportError
    from Cython.Compiler.Version import version
    version = tuple([int(v) \
        for v in re.split("[^\d]", version) if v.isdigit()])
    if version < (0, 17, 1):
        print "Your Cython version is too old"
        raise ImportError
except ImportError:
    source_suffix = '.c'
    cythonize = lambda x:x
    print "No Cython, will compile from .c files"
    for cmd in extra_commands:
        if cmd in sys.argv:
            print "'%s' command not available without Cython" % cmd
            sys.exit(1)
else:
    from Cython.Build import cythonize
    source_suffix = '.pyx'


# Save some repetitive typing.  We have all compiled modules in place
# with their python siblings, so their paths and names are the same.
def CythonExtension(module_name, **kw):
    path = module_name.replace('.', '/')
    return Extension(module_name, [path + source_suffix], **kw)


short_description = "COmparative GENomics Toolkit"

# This ends up displayed by the installer
long_description = """Cogent
A toolkit for statistical analysis of biological sequences.
Version %s.
""" % __version__

setup(
    name="cogent",
    version=__version__,
    url="http://github.com/pycogent/pycogent",
    author="Gavin Huttley, Rob Knight",
    author_email="gavin.huttley@anu.edu.au, rob@spot.colorado.edu",
    description=short_description,
    long_description=long_description,
    platforms=["any"],
    license=["GPL"],
    keywords=["biology", "genomics", "statistics", "phylogeny", "evolution",
                "bioinformatics"],
    classifiers=[
            "Development Status :: 5 - Production/Stable",
            "Intended Audience :: Science/Research",
            "License :: OSI Approved :: GNU General Public License (GPL)",
            "Topic :: Scientific/Engineering :: Bio-Informatics",
            "Topic :: Software Development :: Libraries :: Python Modules",
            "Operating System :: OS Independent",
            ],
    packages=['cogent', 'cogent.align', 'cogent.align.weights', 'cogent.app',
                'cogent.cluster', 'cogent.core', 'cogent.data', 'cogent.db',
                'cogent.db.ensembl', 'cogent.draw',
                'cogent.evolve', 'cogent.format', 'cogent.maths',
                'cogent.maths.matrix', 'cogent.maths.stats',
                'cogent.maths.stats.cai', 'cogent.maths.unifrac',
                'cogent.maths.spatial', 'cogent.motif', 'cogent.parse',
                'cogent.phylo', 'cogent.recalculation', 'cogent.seqsim',
                'cogent.struct', 'cogent.util'],
    ext_modules=cythonize([
        CythonExtension("cogent.align._compare"),
        CythonExtension("cogent.align._pairwise_seqs"),
        CythonExtension("cogent.align._pairwise_pogs"),
        CythonExtension("cogent.evolve._solved_models"),
        CythonExtension("cogent.evolve._likelihood_tree"),
        CythonExtension("cogent.evolve._pairwise_distance"),
        CythonExtension("cogent.struct._asa"),
        CythonExtension("cogent.struct._contact"),
        CythonExtension("cogent.maths._period"),
        CythonExtension("cogent.maths.spatial.ckd3"),
    ]),
    include_dirs = [numpy_include_dir],
    cmdclass = extra_commands,
    extras_require={"mysql": ["PyMySQL", "sqlalchemy"],
                    "mpi": ["mpi4py"],
                    "all": ["PyMySQL", "sqlalchemy", "matplotlib", "mpi4py"]},
)



#!/usr/bin/env python
"""Application controller for RTAX version 1.0

Includes application controller for RTAX.

Modified from uclust.py and rdp_classifier.py on 12-27-11
"""

__author__ = "David Soergel"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["David Soergel", "William Walters", "Greg Caporaso"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "David Soergel"
__email__ = "soergel@cs.umass.edu"
__status__ = "Production"

from os import remove, makedirs
from os.path import exists, split, splitext, basename, isdir, abspath, isfile
from cogent.parse.fasta import MinimalFastaParser
from cogent.app.parameters import ValuedParameter, FlagParameter
from cogent.app.util import CommandLineApplication, ResultPath,\
 get_tmp_filename, ApplicationError, ApplicationNotFoundError
from cogent.util.misc import remove_files, app_path
from cogent import DNA
import tempfile
import os.path
import re
from sys import stderr

from shutil import rmtree

class RtaxParseError(Exception):
    pass

class Rtax(CommandLineApplication):
    """ Rtax ApplicationController

    """

    _command = 'rtax'
    _input_handler = '_input_as_parameters'
    _parameters = {\

        # -r a reference database in FASTA format
        '-r':ValuedParameter('-',Name='r',Delimiter=' ', IsPath=True),

        # -t a taxonomy file with sequence IDs matching the reference database
        '-t':ValuedParameter('-',Name='t',Delimiter=' ', IsPath=True),

        # -a a FASTA file containing query sequences (single-ended, read 1, or paired-end delimited)
        '-a':ValuedParameter('-',Name='a',Delimiter=' ', IsPath=True),

        # -b a FASTA file containing query sequences (read 2, with matching IDs)
        '-b':ValuedParameter('-',Name='b',Delimiter=' ', IsPath=True),

        # -l a text file containing sequence IDs to process, one per line
        '-l':ValuedParameter('-',Name='l',Delimiter=' ', IsPath=True),

        # -d a delimiter separating the two reads when provided in a single file
        '-d':ValuedParameter('-',Name='d',Delimiter=' ', IsPath=False, Quote="\""),

        # -i a regular expression used to select part of the fasta header to use as the sequence id.
        '-i':ValuedParameter('-',Name='i',Delimiter=' ', IsPath=False, Quote="'"),

        # -o output file name for classification assignment
        '-o': ValuedParameter('-', Name='o', Delimiter=' ', IsPath=True),

        # -m temporary directory
        '-m': ValuedParameter('-', Name='m', Delimiter=' ', IsPath=True),

        # -f allow fallback from paired-end to single-ended classification when one read is missing
        '-f':FlagParameter(Prefix='-',Name='f'),

        # -g do not allow fallback from paired-end to single-ended classification when one read is too generic
        '-g':FlagParameter(Prefix='-',Name='g')
    }

    _suppress_stdout = False
    _suppress_stderr = False

    #def __init__(self):
    #    super().__init__()...
    #    usearch_command = "usearch"
    #    if not (exists(usearch_command) or app_path(usearch_command)):
    #        raise ApplicationNotFoundError,\
 	#        "Cannot find %s. Is it installed? Is it in your path?"\
 	#        % usearch_command


    def _input_as_parameters(self,data):
        """ Set the input path (a fasta filepath)
        """
        # The list of values which can be passed on a per-run basis
        allowed_values = ['-r','-t','-a','-b','-l','-d','i','-o','-m','-v','-f', '-g']

        unsupported_parameters = set(data.keys()) - set(allowed_values)
        if unsupported_parameters:
            raise ApplicationError,\
             "Unsupported parameter(s) passed when calling rtax: %s" %\
              ' '.join(unsupported_parameters)

        for v in allowed_values:
            # turn the parameter off so subsequent runs are not
            # affected by parameter settings from previous runs
            self.Parameters[v].off()
            if v in data:
                # turn the parameter on if specified by the user
                self.Parameters[v].on(data[v])

        return ''

    def _get_result_paths(self,data):
        """ Return a dict of ResultPath objects representing all possible output
        """
        assignment_fp = str(self.Parameters['-o'].Value).strip('"')
        if not os.path.isabs(assignment_fp):
            assignment_fp = os.path.relpath(assignment_fp, self.WorkingDir)
        return {'Assignments': ResultPath(assignment_fp, IsWritten=True)}



    def _accept_exit_status(self,exit_status):
        """ Test for acceptable exit status

            uclust can seg fault and still generate a parsable .uc file
            so we explicitly check the exit status

        """
        return exit_status == 0

    def getHelp(self):
        """Method that points to documentation"""
        help_str =\
        """
        RTAX is hosted at:
        http://dev.davidsoergel.com/rtax/

        The following paper should be cited if this resource is used:

        Soergel D.A.W., Dey N., Knight R., and Brenner S.E.  2012.
        Selection of primers for optimal taxonomic classification
        of environmental 16S rRNA gene sequences.  ISME J (6), 1440-1444
        """
        return help_str

def assign_taxonomy(dataPath, reference_sequences_fp, id_to_taxonomy_fp, read_1_seqs_fp, read_2_seqs_fp, single_ok=False, no_single_ok_generic=False,
                    header_id_regex=None, read_id_regex = "\S+\s+(\S+)", amplicon_id_regex = "(\S+)\s+(\S+?)\/",
                    output_fp=None, log_path=None, HALT_EXEC=False, base_tmp_dir = '/tmp'):
    """Assign taxonomy to each sequence in data with the RTAX classifier

        # data: open fasta file object or list of fasta lines
        dataPath: path to a fasta file

        output_fp: path to write output; if not provided, result will be
         returned in a dict of {seq_id:(taxonomy_assignment,confidence)}
    """

    usearch_command = "usearch"
    if not (exists(usearch_command) or app_path(usearch_command)):
        raise ApplicationNotFoundError,\
         "Cannot find %s. Is it installed? Is it in your path?"\
         % usearch_command

    my_tmp_dir = get_tmp_filename(tmp_dir=base_tmp_dir,prefix='rtax_',suffix='',result_constructor=str)
    os.makedirs(my_tmp_dir)


    try:
        # RTAX classifier doesn't necessarily preserve identifiers
        # it reports back only the id extracted as $1 using header_id_regex
        # since rtax takes the original unclustered sequence files as input,
        # the usual case is that the regex extracts the amplicon ID from the second field



        # Use lookup table
        read_1_id_to_orig_id = {}
        readIdExtractor = re.compile(read_id_regex)  # OTU clustering produces ">clusterID read_1_id"
        data = open(dataPath,'r')
        for seq_id, seq in MinimalFastaParser(data):
            # apply the regex
            extract = readIdExtractor.match(seq_id)
            if extract is None:
                stderr.write("Matched no ID with read_id_regex " + read_id_regex +" in '" + seq_id + "' from file " + dataPath + "\n")
            else:
                read_1_id_to_orig_id[extract.group(1)] = seq_id
                #stderr.write(extract.group(1) + " => " +  seq_id + "\n")
            #seq_id_lookup[seq_id.split()[1]] = seq_id
        data.close()



        # make list of amplicon IDs to pass to RTAX

        id_list_fp = open(my_tmp_dir+"/ampliconIdsToClassify", "w")

        # Establish mapping of amplicon IDs to read_1 IDs
        # simultaneously write the amplicon ID file for those IDs found in the input mapping above

        amplicon_to_read_1_id = {}
        ampliconIdExtractor = re.compile(amplicon_id_regex)  # split_libraries produces >read_1_id ampliconID/1 ...  // see also assign_taxonomy 631
        read_1_data = open(read_1_seqs_fp,'r')
        for seq_id, seq in MinimalFastaParser(read_1_data):
            # apply the regex
            extract = ampliconIdExtractor.match(seq_id)
            if extract is None:
                stderr.write("Matched no ID with amplicon_id_regex " + amplicon_id_regex + " in '" + seq_id + "' from file " + read_1_seqs_fp + "\n")
            else:
                read_1_id = extract.group(1)
                amplicon_id = extract.group(2)
                try:
                    amplicon_to_read_1_id[amplicon_id] = read_1_id
                    bogus = read_1_id_to_orig_id[read_1_id]  # verify that the id is valid
                    id_list_fp.write('%s\n' % (amplicon_id))
                except KeyError:
                    pass
        data.close()
        id_list_fp.close()

        app = Rtax(HALT_EXEC=HALT_EXEC)

        temp_output_file = tempfile.NamedTemporaryFile(
            prefix='RtaxAssignments_', suffix='.txt')
        app.Parameters['-o'].on(temp_output_file.name)
        app.Parameters['-r'].on(reference_sequences_fp)
        app.Parameters['-t'].on(id_to_taxonomy_fp)
        # app.Parameters['-d'].on(delimiter)
        app.Parameters['-l'].on(id_list_fp.name)  # these are amplicon IDs
        app.Parameters['-a'].on(read_1_seqs_fp)
        if read_2_seqs_fp is not None:
            app.Parameters['-b'].on(read_2_seqs_fp)
        app.Parameters['-i'].on(header_id_regex)
        app.Parameters['-m'].on(my_tmp_dir)
        if single_ok: app.Parameters['-f'].on();
        if no_single_ok_generic: app.Parameters['-g'].on();
        #app.Parameters['-v'].on()

        app_result = app()

        if log_path:
            f=open(log_path, 'a')
            errString=''.join(app_result['StdErr'].readlines()) + '\n'
            f.write(errString)
            f.close()

        assignments = {}

        # restore original sequence IDs with spaces

        for line in app_result['Assignments']:
            toks = line.strip().split('\t')
            rtax_id = toks.pop(0)
            if len(toks):
                bestpcid = toks.pop(0)  # ignored
            lineage = toks

            # RTAX does not provide a measure of confidence.  We could pass one in,
            # based on the choice of primers, or even look it up on the fly in the tables
            # from the "optimal primers" paper; but it would be the same for every
            # query sequence anyway.
            # we could also return bestpcid, but that's not the same thing as confidence.
            confidence = 1.0

            read_1_id = amplicon_to_read_1_id[rtax_id]
            orig_id = read_1_id_to_orig_id[read_1_id]
            if lineage:
                assignments[orig_id] = (';'.join(lineage), confidence)
            else:
                assignments[orig_id] = ('Unclassified', 1.0)

        if output_fp:
            try:
                output_file = open(output_fp, 'w')
            except OSError:
                raise OSError("Can't open output file for writing: %s" % output_fp)
            for seq_id, assignment in assignments.items():
                lineage, confidence = assignment
                output_file.write(
                    '%s\t%s\t%1.3f\n' % (seq_id, lineage, confidence))
            output_file.close()
            return None
        else:
            return assignments
    finally:
        try:
            rmtree(my_tmp_dir)
        except OSError:
            pass



#!/usr/bin/env python

"""Tests Filter, Organizer and filterfunctions, classes for filtering
"""

from cogent.util.organizer import Filter, Organizer, GroupList, regroup
from cogent.util.transform import find_any, find_no, find_all,\
    keep_if_more, exclude_if_more, keep_if_more_other, exclude_if_more_other
from cogent.util.unit_test import TestCase,main

__author__ = "Sandra Smit"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Sandra Smit", "Rob Knight"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Sandra Smit"
__email__ = "sandra.smit@colorado.edu"
__status__ = "Production"

class Sequence(object):
    """Simple sequence class for tests."""
    def __init__(self, s, info):
        self.s = s
        self.__dict__.update(info)
        if not hasattr(self, 'Gene'):
            self.Gene = None
    def __contains__(self, s):
        return s in self.s
    def __repr__(self):
        return `self.s`
    def __iter__(self):
        return iter(self.s)
    def __nonzero__(self):
        return bool(self.s)
    def lower(self):
        return self.s.lower()
    def __cmp__(self, other):
        return cmp(self.s,other.s)

class FilterTests(TestCase):
    """Tests of Filter class"""

    def test_init(self):
        """Filter should init as expected"""
        empty_filter = Filter('',{})
        named_empty_filter = Filter('Archaea',{})

        self.assertEqual(empty_filter,{})
        self.assertEqual(empty_filter.Name,'')
        self.assertEqual(named_empty_filter,{})
        self.assertEqual(named_empty_filter.Name,'Archaea')
        
        f = find_all('abcd')
        g = keep_if_more_other('ab',7)
        fil = Filter('Archaea',{'Arch':[f,g]})
        assert fil['Arch'][0] is f
        assert fil['Arch'][1] is g

    def test_call_empty(self):
        """Empty Filter should return True when called on anything"""
        f = Filter('',{})
        data = ['aa','bb','cc']
        self.assertEqual(f(data),True)
    
    def test_call_full(self):
        """Filter should return True if the object satisfies all criteria"""
        seq1 = Sequence('ACGU',{'Gene':'LSU'})
        seq2 = Sequence('ACGUACGU',{'Gene':'SSU'})
        seq3 = Sequence('ACGUN',{'Gene':'LSU'})
        seq4 = Sequence('ACG',{'Gene':'LSU'})
        seq5 = Sequence('ACGU',{})
        seq6 = Sequence('',{})
        
        f = Filter('valid',{None:[find_all('AGCU'),find_no('N')],\
                            'Gene':[find_any(['LSU'])]})
        self.assertEqual(f(seq1),True)
        self.assertEqual(f(seq2),False)
        self.assertEqual(f(seq3),False)
        self.assertEqual(f(seq4),False)
        self.assertEqual(f(seq5),False)
        self.assertEqual(f(seq6),False)
        
class GroupListTests(TestCase):
    """Tests of GroupList class"""

    def test_init_empty(self):
        """Empty GroupList should init OK"""
        g = GroupList([])
        self.assertEqual(len(g),0)
        self.assertEqual(g.Groups,[])

    def test_init_full(self):
        """GroupList should init OK with data and groups"""
        data = ['a','b','c']
        groups = [1,2,3]
        g = GroupList(data,groups)
        self.assertEqual(g,data)
        self.assertEqual(g.Groups,groups)
        self.assertEqual(len(g),3)

        
class OrganizerTests(TestCase):
    """Tests of Classifier class"""

    def setUp(self):
        """Define some standard Organizers for testing"""
        self.Empty = Organizer([])
        
        self.a = Filter('a',{None:[find_any('a')]})
        self.b = Filter('b',{None:[find_any('b')]})
        self.Ab_org = Organizer([self.a,self.b])
        
        lsu = Filter('LSU',{None:[exclude_if_more('N',5)],\
                            'Gene':[find_any(['LSU'])]})
        ssu = Filter('SSU',{None:[exclude_if_more('N',5)],\
                            'Gene':[find_any(['SSU'])]})
        self.Gene_org = Organizer([lsu,ssu])
        
        self.Ab_seq = ['aa','bb','abab','cc','']
        self.seq1 = Sequence('ACGU',{'Gene':'LSU'})
        self.seq2 = Sequence('ACGUACGU',{'Gene':'SSU'})
        self.seq3 = Sequence('ACGUNNNNNN',{'Gene':'LSU'})
        self.seq4 = Sequence('ACGUNNNNNN',{'Gene':'SSU'})
        self.seq5 = Sequence('ACGU',{})
        self.seq6 = Sequence('',{})
        self.seq7 = Sequence('ACGU',{'Gene':'unit'})
        
        self.Gene_seq = [self.seq1,self.seq2,self.seq3,self.seq4,\
                self.seq5,self.seq6,self.seq7]

        f = Filter('valid',{None:[find_all('AGCU'),find_no('N')],\
                            'Gene':[find_any(['LSU'])]})
        self.Mult_func_org = Organizer([f])
    
    def test_init_empty(self):
        """Empty Organizer should init correctly"""
        org = self.Empty
        self.assertEqual(len(org),0)

    def test_init_full(self):
        """Organizer should init correctly with multiple functions"""
        org = Organizer([self.a,self.b])
        self.assertEqual(org[0],self.a)
        self.assertEqual(org[1],self.b)
        self.assertEqual(len(org),2)

    def test_empty_org_empty_list(self):
        """Empty Organizer should return [] when applied to []"""
        org = self.Empty
        l = []
        self.assertEqual(org(l),[])

    def test_empty_org_full_list(self):
        """Empty organizer, applied to full list, should return the original"""
        org = self.Empty
        l = self.Ab_seq
        obs = org(l)
        self.assertEqual(obs,[l])
        self.assertEqual(obs[0].Groups,[None])

    def test_full_org_empty_list(self):
        """Organizer should return [] when applied to []"""
        org = self.Ab_org
        l = []
        obs = org(l)
        self.assertEqual(obs,[])

    def test_full_org_full_list(self):
        """Organizer should return correct organization"""
        org = self.Ab_org
        l = self.Ab_seq
        obs = org(l)
        obs.sort()
        exp = [['aa','abab'],['bb'],['cc','']]
        self.assertEqual(obs,exp)
        self.assertEqual(obs[0].Groups,['a'])
        self.assertEqual(obs[1].Groups,['b'])
        self.assertEqual(obs[2].Groups,[None])

    def test_double_org_empty_list(self):
        """Organizer should return [] when applied to []"""
        org = self.Gene_org
        l = []
        obs = org(l)
        self.assertEqual(obs,[])

    def test_double_org_full_list(self):
        """Organizer should handle multiple filters correctly"""
        org = self.Gene_org
        l = self.Gene_seq
        obs = org(l)
        obs.sort()
        exp = [[self.seq1],[self.seq2],[self.seq3,self.seq4,\
                    self.seq5,self.seq6,self.seq7]]
        self.assertEqual(obs,exp)
        self.assertEqual(obs[0].Groups,['LSU'])
        self.assertEqual(obs[1].Groups,['SSU'])
        self.assertEqual(obs[2].Groups,[None])

    def test_multiple_func(self):
        """Organizer should handle filter with multiple functions correctly"""
        org = self.Mult_func_org
        l = self.Gene_seq
        obs = org(l)
        obs.sort()
        exp = [[self.seq1],[self.seq2,self.seq3,self.seq4,self.seq5,\
                    self.seq6,self.seq7]]
        self.assertEqual(obs,exp)
        self.assertEqual(obs[0].Groups,['valid'])
        self.assertEqual(obs[1].Groups,[None])


class organizerTests(TestCase):
    """Tests for module-level functions"""

    def test_regroup(self):
        """regroup: should groups with identical hierarchy-info together"""
        g1 = GroupList([1,2,3],['a'])
        g2 = GroupList([4,5,6],['b'])
        g3 = GroupList([7,7,7],['a','b'])
        g4 = GroupList([8,8,8],['a'])
        all = [g1, g2, g3, g4]
        self.assertEqualItems(regroup(all), [[1,2,3,8,8,8],[7,7,7],[4,5,6]])


        
if __name__ == "__main__":
    main()   





from numpy import identity, zeros, inner, allclose, array, asarray, maximum, exp
from numpy.linalg import inv, eig

import cogent.maths.matrix_exponentiation as cme

__author__ = "Ben Kaehler"
__copyright__ = "Copyright 2007-2014, The Cogent Project"
__credits__ = ['Ben Kaehler', 'Von Bing Yap']
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Ben Kaehler"
__email__ = "benjamin.kaehler@anu.edu.au"
__status__ = "Production"


class _Exponentiator(object):
    def __init__(self, Q):
        self.Q = Q

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, repr(self.Q))

class VanLoanIntegratingExponentiator(_Exponentiator):
    
    """An exponentiator that evaluates int_0^t exp(Q*s)ds * R
    using the method of Van Loan [1]. Complexity is that of the Exponentiator.
    [1] Van Loan, C. F. (1978). Computing integrals involving the matrix
    exponential. IEEE Trans. Autmat. Control 23(3), 395-404."""

    def __init__(self, Q, R=None, exponentiator=cme.RobustExponentiator):
        """
        Q -- an n x n matrix.
        R -- an n x m matrix. Defaults to the identity matrix. Can be a rank-1
        array.
        exponentiator -- Exponentiator used in Van Loan method. Defaults to
        RobustEstimator.
        """
        self.Q = Q
        Qdim = len(Q)
        if R is None:
            self.R = identity(Qdim)
        else:
            if len(R.shape) == 1: # Be kind to rank-1 arrays
                self.R = R.reshape((R.shape[0], 1))
            else:
                self.R = R
        Cdim = Qdim + self.R.shape[1]
        C = zeros((Cdim, Cdim))
        C[:Qdim,:Qdim] = Q
        C[:Qdim,Qdim:] = self.R
        self.expm = exponentiator(C)

    def __call__(self, t=1.0):
        return self.expm(t)[:len(self.Q),len(self.Q):]

class VonBingIntegratingExponentiator(_Exponentiator):

    """An exponentiator that evaluates int_0^t exp(Q*s)ds
    using the method of Von Bing."""

    def __init__(self, Q):
        """ 
        Q -- a diagonisable matrix.
        """
        self.Q = Q
        self.roots, self.evT = eig(Q)
        self.evI = inv(self.evT.T)
        # Remove following check if performance is a concern
        reQ = inner(self.evT*self.roots, self.evI).real
        if not allclose(Q, reQ): 
            raise ArithmeticError, "eigendecomposition failed"

    def __call__(self, t=1.0):
        int_roots = array([t if abs(x.real) < 1e-6 else
            (exp(x*t)-1)/x for x in self.roots])
        result = inner(self.evT * int_roots, self.evI)
        if result.dtype.kind == "c":
            result = asarray(result.real)
        result = maximum(result, 0.0)
        return result




#!/usr/bin/env python

from string import strip
from cogent.parse.fasta import MinimalFastaParser

__author__ = "Jesse Zaneveld"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Jesse Zaneveld", "Rob Knight"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Jesse Zaneveld"
__email__ = "zaneveld@gmail.com"
__status__ = "Production"

"""
Parser for KEGG fasta files 

This code is useful for parsing the KEGG .nuc or .pep files
"""

def parse_fasta(lines):
    """lightweight parser for KEGG FASTA format sequences"""
    for label, seq in MinimalFastaParser(lines):
        yield '\t'.join(list(kegg_label_fields(label)) \
          + [seq] + ["\n"])

def kegg_label_fields(line):
    """Splits line into KEGG label fields.

    Format is species:gene_id [optional gene_name]; description.
    """
    fields = map(strip, line.split(None, 1))
    id_ = fields[0] 
    species, gene_id = map(strip, id_.split(':',1))
    #check if we got a description
    gene_name = description = ''
    if len(fields) > 1:
        description = fields[1]
        if ';' in description:
            gene_name, description = map(strip, description.split(';',1))
    return id_, species, gene_id, gene_name, description



if __name__ == '__main__':
    from sys import argv
    filename = argv[1]
    for result_line in parse_fasta(open(filename)):
        print result_line.strip()



"""Classes and functions for computing and manipulating accessible 
surface areas (ASA)."""

from cogent.app.stride import Stride
from cogent.parse.stride import stride_parser
from cogent.struct.selection import einput
from cogent.struct.annotation import xtradata
from cogent.maths.geometry import sphere_points, coords_to_symmetry, \
                                  coords_to_crystal
from _asa import asa_loop
from numpy import array, r_


__author__ = "Marcin Cieslik"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Marcin Cieslik"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Marcin Cieslik"
__email__ = "mpc4p@virginia.edu"
__status__ = "Development"


def _run_asa(atoms, lattice_coords, spoints, probe=1.4, bucket_size=5, \
             MAXSYM=200000):
    """Runs an ASA calculation. This function takes a selection of atoms 
    (in the most common case all atoms in a structure) and lattice coordinates 
    (in the most common a 3x3 box of unit-cells).
    
    This function should be considered low-level and not part of the interface.
    
    Arguments:
    
        - atoms: Holder of atom entities.
        - lattice_coords: Numpy array of coordinates.
        - spoints: an array of coordinates on the unit sphere, defines the 
          accuracy of ASA calculation.
        - probe: size of the probe i.e. solvent molecule.
        - bucket_size: see: ``KDTree``.
        - MAXSYM (int): maximum number of symmetry generated atoms.
    """
    # get array of radii inflated by probe size of the selection of atoms.
    atom_radii = array(atoms.getData('radius', forgiving=False)) + probe
    # get array of coordinates
    atom_coords = array(atoms.getData('coords', forgiving=False))
    # calculate bounding box in a form of an array
    search_limit = 2 * (2.0 + probe)    # 2.0 is maximum atom radius
    atom_box = r_[atom_coords.min(axis=0) - search_limit, \
                  atom_coords.max(axis=0) + search_limit]
    # the lattice coordinates are atom coordinates after transformations in a 
    # 4D-array this array gets reshaped into an all_atoms x 3 array.
    shape = lattice_coords.shape
    lattice_coords = \
        lattice_coords.reshape((shape[0] * shape[1] * shape[2], shape[3]))
    # this calls the cython code which loops over all query atoms, surface 
    # points, and lattice atoms
    return asa_loop(atom_coords, lattice_coords, atom_radii, atom_radii, \
                    spoints, atom_box, probe, bucket_size, MAXSYM)

def _prepare_entities(entities):
    """Prepares input entities for ASA calculation, which includes masking water
    molecules and water chains.
    """
    # First we mask all water residues and chains with all residues masked 
    # (water chains).
    lattice_residues = einput(entities, 'R')
    lattice_residues.maskChildren('H_HOH', 'eq', 'name')
    lattice_chains = einput(entities, 'C')
    lattice_chains.maskChildren([], 'eq', 'values', method=True)
    # if no residues or chains are left - no atoms to work with, 
    # abort with warning.
    if not lattice_chains.values():
        # the following makes sure that masking changes by the above
        # tests are reverted.
        lattice_structures = einput(entities, 'S')
        lattice_structures.setUnmasked(force=True)
        raise ValueError('No unmasked atoms to build lattice.')
    # these are all atoms we can work with
    lattice_atoms = einput(entities, 'A')
    lattice_atoms.dispatch('setRadius')

def _postpare_entities(entities):
    """Restores entities after ASA calculation, which includes unmasking."""
    structures = einput(entities, 'S')
    structures.setUnmasked(force=True)


def _prepare_asa(entities, symmetry_mode=None, crystal_mode=None, points=960, \
                **kwargs):
    """Prepares the atomic solvent-accessible surface area (ASA) calculation.
    
    Arguments:
    
        - entities: input entities for ASA calculation (most commondly a 
          structure entity).
        - symmetry_mode (str): One of 'uc', 'bio' or 'table'. This defines the 
          transformations of applied to the coordinates of the input entities. 
          It is one of 'bio', 'uc' or 'table'. Where 'bio' and 'uc' are 
          transformations to create the biological molecule or unit-cell from 
          the PDB header. The 'table' uses transformation matrices derived from 
          space-group information only using crystallographic tables(requires 
          ``cctbx``).
        - crystal_mode (int): Defines the number of unit-cells to expand the 
          initial unit-cell into. The number of unit-cells in each direction 
          i.e. 1 is makes a total of 27 unit cells: (-1, 0, 1) == 3, 3^3 == 27
        - points: number of points on atom spheres higher is slower but more 
          accurate.    

    Additional keyworded arguments are passed to the ``_run_asa`` function.
    """
    # generate uniform points on the unit-sphere
    spoints = sphere_points(points)
    # prepare entities for asa calculation
    # free-floating area mode
    result = {}
    atoms = einput(entities, 'A')
    if not symmetry_mode and not crystal_mode:

        coords = array(atoms.getData('coords', forgiving=False))
        coords = array([[coords]]) # fake 3D and 4D
        idx_to_id = dict(enumerate(atoms.getData('getFull_id', \
                                                forgiving=False, method=True)))
        asas = _run_asa(atoms, coords, spoints, **kwargs)
        for idx in xrange(asas.shape[0]):
            result[idx_to_id[idx]] = asas[idx]
    # crystal-contact area mode    
    elif symmetry_mode in ('table', 'uc'):
        structure = einput(entities, 'S').values()[0]
        sh = structure.header
        coords = array(atoms.getData('coords', forgiving=False))
        idx_to_id = dict(enumerate(atoms.getData('getFull_id', \
                                                forgiving=False, method=True)))
        # expand to unit-cell, real 3D
        coords = coords_to_symmetry(coords, \
                                            sh[symmetry_mode + '_fmx'], \
                                            sh[symmetry_mode + '_omx'], \
                                            sh[symmetry_mode + '_mxs'], \
                                            symmetry_mode)
        # expand to crystal, real 4D
        if crystal_mode:
            coords = coords_to_crystal(coords, \
                                               sh[symmetry_mode + '_fmx'], \
                                               sh[symmetry_mode + '_omx'], \
                                               crystal_mode) # real 4D
        else:
            coords = array([coords]) # fake 4D
        asas = _run_asa(atoms, coords, spoints, **kwargs)
        for idx in xrange(asas.shape[0]):
            result[idx_to_id[idx]] = asas[idx]

     # biological area mode
    elif symmetry_mode == 'bio':
        structure = einput(entities, 'S').values()[0]
        chains = einput(entities, 'C')
        sh = structure.header
        start = 0
        for chain_ids, mx_num in sh['bio_cmx']:
            sel = chains.selectChildren(chain_ids, 'contains', 'id').values()
            atoms = einput(sel, 'A')
            coords = array(atoms.getData('coords', forgiving=False))
            idx_to_id = dict(enumerate(atoms.getData('getFull_id', \
                                              forgiving=False, method=True)))
            stop = start + mx_num
            coords = coords_to_symmetry(coords, \
                                               sh['uc_fmx'], \
                                               sh['uc_omx'], \
                                               sh['bio_mxs'][start:stop], \
                                               symmetry_mode)
            coords = array([coords])
            start = stop
            asas = _run_asa(atoms, coords, spoints, **kwargs)
            for idx in xrange(asas.shape[0]):
                result[idx_to_id[idx]] = asas[idx]
    return result


def asa_xtra(entities, mode='internal', xtra_key=None, **asa_kwargs):
    """Calculates accessible surface areas (ASA) and puts the results into the
    xtra dictionaries of entities.
    
    Arguments:
    
        - entities: an entity or sequence of entities
        - mode(str): 'internal' for calculations using the built-in cython code 
          or 'stride' if the stride binary should be called to do the job.
        - xtra_key(str): Key in the xtra dictionary to hold the result for each
          entity
    
    Additional keyworded arguments are passed to the ``_prepare_asa`` and 
    ``_run_asa`` functions.
    """
    xtra_key = xtra_key or 'ASA'
    structures = einput(entities, 'S')
    if len(structures.values()) > 1:
        raise ValueError('Entities from multiple structures are not supported.')
    if mode == 'internal':
        _prepare_entities(entities) # mask waters
        result = _prepare_asa(entities, **asa_kwargs) # calculate ASA
        _postpare_entities(entities) # unmask waters
        result = dict([(id, {xtra_key:v}) for id, v in result.iteritems()])
        xtradata(result, structures)
    elif mode == 'stride':
        models = einput(entities, 'M')
        stride_app = Stride()
        result = stride_app(entities)['StdOut'].readlines()
        result = stride_parser(result)
        xtradata(result, structures.values()[0][(0,)])
    else:
        raise ValueError('Not a valid mode: "%s"' % mode)
    return result



#!/usr/bin/env python
"""Parser for PSL format (default output by blat).
   Compatible with blat v.34
"""

from cogent import LoadTable
from cogent.parse.table import ConvertFields

__author__ = "Gavin Huttley, Anuj Pahwa"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Rob Knight","Peter Maxwell", "Gavin Huttley", "Anuj Pahwa"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Development"

def make_header(lines):
    """returns one header line from multiple header lines"""
    lengths = map(len, lines)
    max_length = max(lengths)
    for index, line in enumerate(lines):
        if lengths[index] != max_length:
            for i in range(lengths[index], max_length):
                line.append('')
    
    header = []
    for t, b in zip(*lines):
        if t.strip().endswith('-'):
            c = t.strip()+b
        else:
            c = ' '.join([t.strip(), b.strip()])
        header += [c.strip()]
    return header

int_series = lambda x: map(int, x.replace(',',' ').split())

row_converter = ConvertFields([(i, int) for i in range(8)]+\
                              [(i, int) for i in range(10, 13)]+\
                              [(i, int) for i in range(14, 18)]+\
                              [(i, int_series) for i in range(18, 21)])

def MinimalPslParser(data, row_converter=row_converter):
    """returns version, header and rows from data"""
    if type(data) == str:
        data = open(data)
    
    psl_version = None
    header = None
    rows = []
    
    for record in data:
        if psl_version is None:
            assert 'psLayout version' in record
            psl_version = record.strip()
            yield psl_version
            continue
        
        if not record.strip():
            continue
        
        if header is None and record[0] == '-':
            header = make_header(rows)
            yield header
            rows = []
            continue
        
        rows += [record.rstrip().split('\t')]
        if header is not None:
            yield row_converter(rows[0])
            rows = []
        
    

def PslToTable(data):
    """converts psl format to a table"""
    parser = MinimalPslParser(data)
    version = parser.next()
    header = parser.next()
    rows = [row for row in parser]
    table = LoadTable(header=header, rows=rows, title=version)
    return table



#!/usr/bin/env python
""" Provides tests for EbiParser and related classes and functions.
"""

from cogent.parse.ebi import cc_alternative_products_parser, \
    cc_basic_itemparser, cc_biophysicochemical_properties_parser, \
    cc_interaction_parser, cc_itemfinder, cc_parser, EbiFinder, \
    MinimalEbiParser, hanging_paragraph_finder, join_parser, \
    join_split_dict_parser, join_split_parser, labeloff, linecode_maker, \
    linecode_merging_maker, mapping_parser, pairs_to_dict, period_tail_finder, \
    rstrip_, ft_basic_itemparser, ft_id_parser, ft_mutagen_parser, \
    ft_mutation_parser, ft_parser, try_int, ra_parser, rc_parser, \
    rg_parser, rl_parser, rn_parser, rp_parser, rt_parser, rx_parser, \
    gn_parser, single_ref_parser, ac_parser, de_itemparser, dr_parser, \
    dt_parser, id_parser, oc_parser, og_parser, os_parser, ox_parser, \
    sq_parser, de_parser, RecordError, FieldError, curry, required_labels, \
    EbiParser, pr_parser
from cogent.util.unit_test import TestCase, main
from cogent.core.sequence import Sequence
from cogent.core.info import Info

__author__ = "Zongzhi Liu"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Zongzhi Liu", "Rob Knight"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Zongzhi Liu"
__email__ = "zongzhi.liu@gmail.com"
__status__ = "Development"

def item_empty_filter(d):
    """return a dict with only nonempty values"""
    pairs = [(k,v) for (k,v) in d.iteritems() if v]
    return dict(pairs)
                    
class EbiTests(TestCase):
    """ Tests ebi parsers and generic parsers and general functions """
    def setUp(self):
        """ Construct some fake data for testing purposes """
        pass
    def test_item_empty_filter(self):
        """item_empty_filter: known values"""
        inputs = [
            {1:0, 2:1, 3:'', 4:[], 5:False, 6:{}}]
        expects = [
            {2:1}]
        self.assertEqual(map(item_empty_filter, inputs), expects)
    def test_rstrip_(self):
        """rstrip_ should generate the expected function"""
        test = ' aaa;  '
        self.assertEqual(rstrip_('; ')(test),
                test.rstrip('; '))
        #test default
        self.assertEqual(rstrip_()(test),
                test.rstrip())
    
    def test_hanging_paragraph_finder(self):
        """hanging_paragraph_finder should give expected results"""
        f = hanging_paragraph_finder
        test = [ 
            '-aaa:',
            '  content',
            '    content',
            '-bbb:',  #3
            '  bbb',
            'c',]
        self.assertEqual(list(f(test)),
                [test[0:3], test[3:-1], test[-1:]])

        #test all indent lines
        all_indent = ['  aa', '  bb']
        self.assertEqual(list(f(all_indent)), [all_indent])

        #test empty lines
        self.assertEqual(list(f(['',' '])), [])

    def test_period_tail_finder(self):
        """period_tail_finder should yield each group of expected lines."""
        test = "a\naa.\nb\nbb.".splitlines()
        self.assertEqual(list(period_tail_finder(test)),
                [['a','aa.'],['b','bb.']])

    def test_EbiFinder(self):
        """EbiFinder should return expected list"""
        test = "a\n//\nb\n//".splitlines()
        self.assertEqual(list(EbiFinder(test)),
                [['a','//'],['b','//']])

        test_fail = test + ['c']
        self.assertRaises(RecordError, 
                list, EbiFinder(test_fail))
        

    def test_pairs_to_dict(self):
        """pairs_to_dict should return the expected dict"""
        test_dict = {'a': 1, 'b': 2, 'b': 3,}
        sorted_items = [('a', 1), ('b', 2), ('b', 3),] 
        add_one = lambda x: x + 1
        double = lambda x: x*2
        set_zero = lambda x: 0
        handlers ={ 'a': add_one, 'b': double,}
        
        #test default all
        self.assertEqual(pairs_to_dict(sorted_items),
                {'a': 1, 'b': 3})

        #test 'overwrite_value' 
        self.assertEqual(pairs_to_dict(sorted_items, 'overwrite_value'),
                {'a': 1, 'b': 3})

        #test no_duplicated_key, raise 
        self.assertRaises(ValueError, pairs_to_dict,
                sorted_items, 'no_duplicated_key')

        #test always_multi_value
        self.assertEqual(pairs_to_dict(sorted_items, 'always_multi_value'),
                {'a': [1], 'b': [2, 3]})

        #test allow multi_value
        self.assertEqual(pairs_to_dict(sorted_items, 'allow_multi_value'),
                {'a': 1, 'b': [2, 3]})

        #test raise error when  key not found in all_keys 
        f = curry(pairs_to_dict, all_keys=['a','c'])
        self.assertRaises(ValueError, f, sorted_items)

        #test handlers
        sorted_items.append(('c', 4))
        self.assertEqual(pairs_to_dict(sorted_items, handlers=handlers,
            default_handler=set_zero), {'a': 2, 'b': 6, 'c': 0})
        
        #test raise error when  no valid handlers were found 
        f = curry(pairs_to_dict, handlers=handlers)
        self.assertRaises(ValueError, f, sorted_items)

        #test sanity
        test_dict = dict(sorted_items)
        self.assertEqual(pairs_to_dict(test_dict.items()),
                test_dict)

    def test_linecode_maker(self):
        """linecode_maker: should return expected tuple"""
        tests = ['AA   aa.',
                 'BB    bb.',
                 'CC C   cc.',
                 'DD dd.']
        expected_linecodes =[
                'AA', 'BB', 'CC C', 'DD dd.']
        #pprint(map(linecode_maker, tests)) 
        self.assertEqual(map(linecode_maker, tests), 
                zip(expected_linecodes, tests))

    def test_labeloff(self):
        """labeloff: should return expected lines"""
        tests = ['AA   aa.',
                 'BB    bb.',
                 'CC C   cc.',
                 'DD dd.',
                 'EE',
                 '']
        #expects = [line[5:] for line in tests]
        expects = ['aa.', ' bb.', '  cc.', '.','','']
        self.assertEqual(labeloff(tests), expects) 

    def test_join_parser(self):
        """join parser should return expected str."""
        test_list = '"aaa\nbbb \nccc"; \n'.splitlines()
        test_str = 'aaa bb. '
        #test default, list input
        self.assertEqual(join_parser(test_list),
                '"aaa bbb  ccc"')
        #test default, str input
        self.assertEqual(join_parser(test_str),
                'aaa bb')
        #test no strip
        self.assertEqual(join_parser(test_list, chars_to_strip=''),
                '"aaa bbb  ccc"; ')
        #test strip 
        self.assertEqual(join_parser(test_list, chars_to_strip='"; '),
                'aaa bbb  ccc')
        #test empty
        self.assertEqual(join_parser([]),'')
        self.assertEqual(join_parser(['', ' ']),'')
        self.assertEqual(join_parser(''),'')
        
        
    def test_join_split_parser(self):
        """join_split_parser: should return expected"""
        f = join_split_parser
        assertEqual = self.assertEqual
        
        assertEqual(f(['aa; bb;', 'cc.']),
                ['aa', 'bb', 'cc'])
        assertEqual(f(['aa; bb, bbb;', 'cc.'],delimiters=';,'),
                ['aa', ['bb','bbb'], 'cc'])
        #test item_modifer
        assertEqual(f('aa (bb) (cc).', '(', item_modifier=rstrip_(') ')),
                ['aa','bb','cc'])
        assertEqual(f('aa (bb)xx (cc).', '(', item_modifier=rstrip_(') ')),
                ['aa','bb)xx','cc'])

        #test empty
        assertEqual(f([]),[''])
        assertEqual(f(['', ' ']),[''])
        assertEqual(f(''),[''])

    def test_join_split_dict_parser(self):
        """join_split_dict_parser: should return expected"""
        f = join_split_dict_parser
        #test default
        self.assertEqual(f('aa=1; bb=2,3; cc=4 (if aa=1);'),
            {'aa':'1', 'bb': ['2','3'], 'cc': '4 (if aa=1)'})

        self.assertEqual(f('aa=1; bb=2,3; cc=4:5', delimiters=';=,:'),
                {'aa':'1', 'bb': ['2','3'], 'cc': '4:5'})
        
        #test strict=False -> splits without dict() 
        self.assertEqual(f('aa=1; bb.', strict=False), [['aa', '1'], ['bb']])

        #test strict -> raise ValueError
        self.assertRaises(ValueError, f, 'aa=1; bb.')
        self.assertRaises(ValueError, f, 'aa=1; bb=2=3.', ';=')
        self.assertRaises(ValueError, f, '')

    def test_mapping_parser(self):
        """mapping_parser: should return expected dict"""
        fields = [None, 'A', 'B', ('C', int), ('D', float)]
        line = 'blah aa bb; 2 3.1;'

        expect = dict(A='aa', B='bb', C=2, D=3.1)
        self.assertEqual(mapping_parser(line, fields), expect)

        #test more splits -> ignore the last splits
        line_leftover = line + 'blah blah'
        self.assertEqual(mapping_parser(line_leftover, fields), expect)

        #test more fields -> truncate the last fields
        fields_leftover = fields + ['E']
        self.assertEqual(mapping_parser(line, fields_leftover), expect)

        #test empty
        self.assertEqual(mapping_parser('', fields), {})

    def test_linecode_merging_maker(self):
        """linecode_merging_maker: """
        f = linecode_merging_maker
        lines =['ID   id.', 'RN   rn.', 'RR   invalid', 'RN  rn.']
        labels = ['ID', 'REF', 'RR', 'RN  rn.']

        self.assertEqual(map(f, lines), zip(labels, lines))

    def test_parse_header(lines):
        pass

    def test_MinimalEbiParser_valid(self):
        """MinimalEbiParser: integrity of output """
        f = curry(MinimalEbiParser, strict=False)

        #test valid result: sequence, number of records, keys of a header 
        valid_result = list(f(fake_records_valid))
        self.assertEqual(len(valid_result), 2)

        sequence, header = valid_result[0]
        self.assertEqual(sequence, 'aaccppgghhh')
        #the first fake record use only the required labels, the header is
        #deleted of '', which was assigned to sequence
        self.assertEqual(list(sorted(header.keys())),
                list(sorted(required_labels))[1:]) #[1:] to exclude the '' 

        #test selected_labels
        selected_labels = ['ID', 'DE']
        select_result = list(f(fake_records_valid, 
            selected_labels=selected_labels))
        self.assertEqual(list(sorted(select_result[0][1].keys())),
                list(sorted(selected_labels)))

        #test bad record - unknown linecode or wrong line format
        self.assertRaises(ValueError, list, 
                f(fake_records_valid + ['ID   id.', 'RR   not valid.','//']))
        self.assertRaises(ValueError, list, 
                f(fake_records_valid + ['ID   id.', ' RN   bad format.','//']))
        self.assertRaises(ValueError, list, 
                f(fake_records_valid + ['ID   id.', 'RN  bad format.','//']))

        #test bad record - not end with '//'
        self.assertRaises(RecordError, list, f(fake_records_valid +
                ['ID   not end with //', '   seq']))

        #test strict: lacking required linecodes
        #?? How to test the error message?  warn message?
        #the first record, [:-5], is valid even when strict=True.
        the_first_valid = list(f(fake_records_valid[:-5], strict=True))[0]
        #[1] get the header_dict
        self.assertEqual(len(the_first_valid[1]),9) 
        
        self.assertRaises(RecordError, list, 
                f(fake_records_valid, strict=True))

    def test_EbiParser(self):
        """EbiParser: """
        f = curry(EbiParser, strict=False)
        first_valid = fake_records_valid[:-5]

        #test valid
        self.assertEqual(len(list(f(fake_records_valid))), 2)
        #test skipping bad record which strict=False
        #self.assertEqual(len(list(f(fake_records_valid[:-1] +
        #    ['OX   xx=no equal.', '//']))), 1)
        ##test Raise RecordError from parse_head when strict=True
        #self.assertRaises(RecordError, list, f(first_valid[:-1] +
        #    ['OX   xx=no equal.', '//'], strict=True))


class RootParsersKnownValues(TestCase):
    """Test most xx_parsers with known value"""
    def test_id_parser(self):
        """id_parser should return expected dict"""
        id_line = [
            'ID   CYC_BOVIN      STANDARD;      PRT;   104 AA.']
        self.assertEqual( id_parser(id_line),
            {'DataClass': 'STANDARD', 'Length': 104, 'MolType': 'PRT', 
            'EntryName': 'CYC_BOVIN'})

    def test_sq_parser(self):
        """sq_parser should return expected dict"""
        lines = [
            "SQ   SEQUENCE   486 AA;  55639 MW;  D7862E867AD74383 CRC64;"]
        self.assertEqual( sq_parser(lines),
            {'Crc64': 'D7862E867AD74383', 'Length': 486, 
            'MolWeight': 55639})

    def test_ac_parser(self):
        """ac_parser should return expected list"""
        lines = [
            "AC   Q16653; O00713; O00714;",
            "AC   Q92892; Q92893;"]
        self.assertEqual( ac_parser(lines),
                ['Q16653', 'O00713', 'O00714', 'Q92892', 'Q92893'])

    def test_oc_parser(self):   
        """oc_parser should return expected list"""
        lines = [
            "OC   Eukaryota; Metazoa; Chordata;",
            "OC   Mammalia;"]
        self.assertEqual(oc_parser(lines),
                ['Eukaryota', 'Metazoa', 'Chordata', 'Mammalia'])

    def test_dt_parser(self):
        """dt_parser should return expected list of list"""
        lines = \
            "DT   01-AUG-1988 (Rel. 08, Created)\n"\
            "DT   30-MAY-2000 (Rel. 39, Last sequence update)\n"\
            "DT   10-MAY-2005 (Rel. 47, Last annotation update)\n".splitlines()
        self.assertEqual( dt_parser(lines),
                ['01-AUG-1988 (Rel. 08, Created)',
                '30-MAY-2000 (Rel. 39, Last sequence update)',
                '10-MAY-2005 (Rel. 47, Last annotation update)'])
        
    def test_de_itemparser(self):
        """de_itemparser: known values"""
        inputs = [
            ' AAA (aa) ',
            'AAA [xx] (aa)',
            'AAA',
            '',]
        expects = [
            {'OfficalName': 'AAA', 'Synonyms': ['aa']},
            {'OfficalName': 'AAA [xx]', 'Synonyms': ['aa']},
            {'OfficalName': 'AAA', 'Synonyms': []},
            {'OfficalName': '', 'Synonyms': []}]
        #pprint(map(de_itemparser, inputs))
        self.assertEqual(map(de_itemparser, inputs), expects)

    def test_pr_parser(self):
        """pr_parser should return expected list"""
        inpr = "PR   Project:PRJNA38045; Project:PRJNA41539;"
        exp = ['PRJNA38045','PRJNA41539']
        obs = pr_parser(inpr)
        self.assertEqual(obs,exp)
        
    def test_de_parser(self):
        """de_parser should return expected list"""
        inputs = [
            "DE   Annexin [Includes: CCC] [Contains: AAA] (Fragment).",
            "DE   A [Includes: II] (Fragment).",
            "DE   A [Contains: CC].",
            "DE   A (Fragment).",
            "DE   A (aa)."]
        filtered_dicts = [item_empty_filter(de_parser([e])) for e in inputs]
        self.assertEqual(map(len, filtered_dicts), [4, 3, 2, 2, 2])
        
    def test_os_parser(self):   
        """os_parser should return expected list"""
        lines = [
            'OS   Solanum melongena (Eggplant) (Auber-',
            'OS   gine).']
        self.assertEqual(os_parser(lines),
                ['Solanum melongena', 'Eggplant', 'Auber- gine'])
        
        lines = \
            """OS   Escherichia coli.""".splitlines()
        self.assertEqual(os_parser(lines),
                ['Escherichia coli'])

    def test_og_parser(self):   
        """og_parser should return expected list"""
        lines = [
            "OG   XXX; xx.",
            "OG   Plasmid R6-5, Plasmid IncFII R100 (NR1), and",
            "OG   Plasmid IncFII R1-19 (R1 drd-19)."]
        self.assertEqual(og_parser(lines),
            ['XXX; xx', [
            'Plasmid R6-5', 'Plasmid IncFII R100 (NR1)',
            'Plasmid IncFII R1-19 (R1 drd-19)']])

    def test_ox_parser(self):   
        """ox_parser should return expected dict"""
        lines = ["OX   NCBI_TaxID=9606;"]

        self.assertEqual(ox_parser(lines),
                {'NCBI_TaxID': '9606'})

    def test_gn_parser(self):   
        """gn_parser should return expected list of dict"""

        lines = [
            "GN   Name=hns; Synonyms=bglY, cur, topS;",
            "GN   OrderedLocusNames=b1237, c1701, ECs1739;"]
        self.assertEqual(gn_parser(lines),
            [{'Synonyms': ['bglY', 'cur', 'topS'], 
            'OrderedLocusNames': ['b1237', 'c1701', 'ECs1739'],
            'Name': 'hns'}])

        lines = [
            "GN   Name=Jon99Cii; Synonyms=SER1, Ser99Da; ORFNames=CG7877;",
            "GN   and",
            "GN   Name=Jon99Ciii; Synonyms=SER2, Ser99Db; ORFNames=CG15519;"]
        self.assertEqual(gn_parser(lines),
            [{'ORFNames': 'CG7877', 'Synonyms': ['SER1', 'Ser99Da'],
            'Name': 'Jon99Cii'},
            {'ORFNames': 'CG15519', 'Synonyms': ['SER2', 'Ser99Db'],
            'Name': 'Jon99Ciii'}])
        
    def test_dr_parser(self):   
        """dr_parser should return expected dict"""
        lines = dr_lines
        self.assertEqual(dr_parser(lines), dr_expect)

class FT_Tests(TestCase):
    """Tests for FT parsers. """
    def test_ft_basic_itemparser(self):
        """ft_basic_itemparser: known values"""
        inputs = [
            ['DNA_BIND    >102    292'],
            ['CONFLICT    327    327       E -> R (in Ref. 2).'],
            ['PROPEP      ?25     48',
            '                             /FTId=PRO_021449.',],
            ['VARIANT     214    214       V -> I.',
            '                             /FTId=VAR_009122.',],]
        expects = [
            ('DNA_BIND', '>102', 292, ''),
            ('CONFLICT', 327, 327, 'E -> R (in Ref. 2)'),
            ('PROPEP', '?25', 48, '/FTId=PRO_021449'),
            ('VARIANT', 214, 214, 'V -> I. /FTId=VAR_009122')]
        #pprint(map(ft_basic_itemparser, inputs))
        self.assertEqual(map(ft_basic_itemparser, inputs), expects)
    
    def test_try_int(self):
        """try_int: known values"""
        inputs = ['9', '0', '-3', '2.3', '<9', '>9', '?', '?35', '']
        expects = [9,   0,   -3,  '2.3', '<9', '>9', '?', '?35', '']
        self.assertEqual(map(try_int, inputs), expects)

    def test_ft_id_parser(self):
        """ft_id_parser: known values"""
        inputs = [
             '',
             'ddd',
             '/FTId=PRO_021449',
             'V -> I. /FTId=VAR_009122',
             'E -> R (tumor). /FTId=VAR_002343',]
        expects = [
            {'Description': '', 'Id': ''},
            {'Description': 'ddd', 'Id': ''},
            {'Description': '', 'Id': 'PRO_021449'},
            {'Description': 'V -> I', 'Id': 'VAR_009122'},
            {'Description': 'E -> R (tumor)', 'Id': 'VAR_002343'}]

        #pprint(map(ft_id_parser, inputs))
        self.assertEqual(map(ft_id_parser, inputs), expects)

    def test_ft_mutation_parser(self):
        """ft_mutation_parser: known values"""
        inputs = [
             '',
             'ddd',  #should raise error?
             'V -> I. /FTId=xxxxxx',  #should raise error?
             'V -> I',
             'E -> R (tumor)',
             'missing (tumor)', ]
        expects = [
            {'MutateFrom': '', 'Comment': '', 'MutateTo': ''},
            {'MutateFrom': 'ddd', 'Comment': '', 'MutateTo': ''},
            {'MutateFrom': 'V', 'Comment': '', 'MutateTo': 'I. /FTId=xxxxxx'},
            {'MutateFrom': 'V', 'Comment': '', 'MutateTo': 'I'},
            {'MutateFrom': 'E', 'Comment': 'tumor', 'MutateTo': 'R'},
            {'MutateFrom': 'missing ', 'Comment': 'tumor', 'MutateTo': ''}]

        #pprint(map(ft_mutation_parser, inputs))
        self.assertEqual(map(ft_mutation_parser, inputs), expects)
    def test_ft_mutation_parser_raise(self):
        """ft_mutation_parser: raise ValueError"""
        pass

    def test_ft_mutagen_parser(self):
        """ft_mutagen_parser: known values"""
        inputs = [
             'C->R,E,A: Loss of cADPr hydrolas',
             'Missing: Abolishes ATP-binding', ]
        expects = [
            {'Comment': ' Loss of cADPr hydrolas',
             'MutateFrom': 'C', 'MutateTo': 'R,E,A'},
            {'Comment': ' Abolishes ATP-binding',
             'MutateFrom': 'Missing', 'MutateTo': ''}]
        
        #pprint(map(ft_mutagen_parser, inputs))
        self.assertEqual(map(ft_mutagen_parser, inputs), expects)

    def test_ft_id_mutation_parser(self):
        """ft_id_mutation_parser: known values"""
        pass
    def test_ft_parser(self):   
        """ft_parser should return expected dict"""
        lines = ft_lines
        #pprint(ft_parser(lines))
        self.assertEqual(ft_parser(lines), ft_expect)


class CC_Tests(TestCase):
    """tests for cc_parsers. """
    def test_cc_itemfinder_valid(self):
        """cc_itemfinder: yield each expected block.""" 
        #pprint(list(cc_itemfinder(labeloff(cc_lines))))
        input_with_license = labeloff(cc_lines)
        self.assertEqual(len(list(cc_itemfinder(
            input_with_license))), 9) 

        input_without_license = labeloff(cc_lines[:-4])
        self.assertEqual(len(list(cc_itemfinder(
            input_without_license))), 8) 

    def test_cc_itemfinder_raise(self):
        """cc_itemfinder: raise RecordError if license block bad.""" 
        input_with_license_lacking_bottom = labeloff(cc_lines[:-1]) 
        self.assertRaises(FieldError, cc_itemfinder,
            input_with_license_lacking_bottom)

    def test_cc_basic_itemparser(self):
        """cc_basic_itemparser: known results or FieldError"""
        valid_topics = [
            ['-!- topic1: first: line', '    second line'],
            ['-!- topic2: ', '    first line', '   second line'],
            [' topic3: not treated invalid topic format'],]
        expects = [
            ('topic1', ['first: line', 'second line']),
            ('topic2', ['first line', 'econd line']),
            ('topic3', ['not treated invalid topic format']),]

        self.assertEqual(map(cc_basic_itemparser, valid_topics),
                expects)

        bad_topic = ['-!- bad_topic without colon', '    FieldError']
        self.assertRaises(FieldError, cc_basic_itemparser, bad_topic)

    def test_cc_interaction_parser(self):
        """cc_interaction_parser: known values"""
        inputs = [ 
            ['Self; NbExp=1; IntAct=EBI-123485, EBI-123485;',
            'Q9W158:CG4612; NbExp=1; IntAct=EBI-123485, EBI-89895;',
            'Q9VYI0:fne; NbExp=1; IntAct=EBI-123485, EBI-126770;',]]
        expects =[
            [('Self', {'NbExp': '1', 
                'IntAct': ['EBI-123485', 'EBI-123485']}),
            ('Q9W158:CG4612', {'NbExp': '1', 
                'IntAct': ['EBI-123485', 'EBI-89895']}),
            ('Q9VYI0:fne', {'NbExp': '1', 
                    'IntAct': ['EBI-123485', 'EBI-126770']})]]
        self.assertEqual(map(cc_interaction_parser, inputs), expects)
        
    def test_cc_biophysicochemical_properties_parser(self):
        """cc_biophysicochemical_properties_parser: known values"""
        #pprint(cc['BIOPHYSICOCHEMICAL PROPERTIES'])  #topic specific parser
        f = cc_biophysicochemical_properties_parser
        valid_inputs = [
            ['Kinetic parameters:',
            '  KM=98 uM for ATP;',
            '  KM=688 uM for pyridoxal;',
            '  Vmax=1.604 mmol/min/mg enzyme;',
            'pH dependence:',
            '  Optimum pH is 6.0. Active pH 4.5 to 10.5;',]
        ]
        expects = [
            {'Kinetic parameters': {
                'KM': ['98 uM for ATP', '688 uM for pyridoxal'],
                'Vmax': '1.604 mmol/min/mg enzyme'},
            'pH dependence': 'Optimum pH is 6.0. Active pH 4.5 to 10.5'},
        ]
        self.assertEqual(map(f, valid_inputs), expects)

    def test_cc_alternative_products_parser(self):
        """cc_alternative_products_parser: know values"""
        f = cc_alternative_products_parser
        valid_inputs = [
            ['Event=Alternative initiation;'
             '  Comment=Free text;',
            'Event=Alternative splicing; Named isoforms=3;',
            '  Comment=Additional isoforms seem to exist.',
            '  confirmation;',
            'Name=1; Synonyms=AIRE-1;',
            '  IsoId=O43918-1; Sequence=Displayed;',
            'Name=3; Synonyms=AIRE-3,',
            'ai-2, ai-3;',  #broken the hanging_paragraph_finder
            '  IsoId=O43918-3; Sequence=VSP_004089, VSP_004090;',]]
        expects = \
            [[{'Comment': 'Free text', 'Event': 'Alternative initiation'},
              {'Comment': 'Additional isoforms seem to exist. confirmation',
               'Event': 'Alternative splicing',
               'Named isoforms': '3',
               'Names': [{'IsoId': 'O43918-1',
                          'Name': '1',
                          'Sequence': 'Displayed',
                          'Synonyms': 'AIRE-1'},
                         {'IsoId': 'O43918-3',
                          'Name': '3',
                          'Sequence': ['VSP_004089', 'VSP_004090'],
                          'Synonyms': ['AIRE-3', 'ai-2', 'ai-3']}]}]]

        #pprint(map(f,valid_inputs))
        self.assertEqual(map(f, valid_inputs), expects)

    def test_cc_parser(self):
        """cc_parser: known values and raise when strict"""
        cc = cc_parser(cc_lines)
        #pprint(cc)
        #print cc.keys()
        self.assertEqual(list(sorted(cc.keys())), 
                ['ALLERGEN', 'ALTERNATIVE PRODUCTS', 
                'BIOPHYSICOCHEMICAL PROPERTIES', 'DATABASE', 'DISEASE',
                'INTERACTION', 'LICENSE', 'MASS SPECTROMETRY'])

        #test Disease topic (default_handler)
        self.assertEqual(cc['DISEASE'], [
            'Defects in PHKA1 are linked to X-linked muscle glycogenosis '
                '[MIM:311870]',
            'Defects in ABCD1 are the cause of recessive X-linked '
                'adrenoleukodystrophy (X-ALD) [MIM:300100]. X-ALD is a rare '
                'phenotype'
            ])

        #test License (default_handler)
        #pprint(cc['LICENSE'])
        self.assertEqual(cc['LICENSE'], [
            'This SWISS-PROT entry is copyright. It is produced through a '
                'collaboration removed'])
        
        #pprint(cc['DATABASE'])  #join_split_dict
        self.assertEqual(cc['DATABASE'], [{
            'NAME': 'CD40Lbase',
            'NOTE': 'European CD40L defect database (mutation db)',
            'WWW': '"http://www.expasy.org/cd40lbase/"'}])

        #test strict
        cc_lines_with_unknown_topic = ['CC   -!- BLAHBLAH: xxxxx'] + cc_lines
        #pprint(cc_parser(cc_lines_with_unknown_topic))
        self.assertEqual(cc_parser(cc_lines_with_unknown_topic)['BLAHBLAH'],
                ['xxxxx'])
        self.assertRaises(FieldError, cc_parser,
                cc_lines_with_unknown_topic, strict=True)


class ReferenceTests(TestCase):
    """Tests for parsers related to reference blocks"""
    def test_ref_finder(self):
        """ref_finder: should return a list of ref blocks"""
        pass

    def test_refs_parser(self):
        """refs_parser: should return a dict of {RN: ref_dict}"""
        pass
        
    def test_single_ref_parser(self):
        """single_ref_parser: should return the expected dict"""
        fake_ref_block = ['RN   [1]',
            'RP   NUCLEOTIDE',
            'RC   STRAIN=Bristol N2;',
            'RX   PubMed=1113;',
            'RA   Zhang L., Wu S.-L., Rubin C.S.;',
            'RT   "A novel ";',
            'RL   J. Biol. Chem. 276:10.',]
        rn, others = single_ref_parser(fake_ref_block)
        self.assertEqual(rn, 1)
        self.assertEqual(len(others), 6)

        #test strict: lacking required labels
        self.assertEqual(len(single_ref_parser(
            fake_ref_block[:-1], strict=False)[1]), 5)
        self.assertRaises(RecordError, single_ref_parser, fake_ref_block[:-1],
                True)

    def test_ra_parser(self):
        """ra_parser should return expected list"""
        lines = \
            "RA   Galinier A., Bleicher F., Negre D.,\n"\
            "RA   Cozzone A.J., Cortay J.-C.;\n".splitlines()
        self.assertEqual( ra_parser(lines),
            ['Galinier A.', 'Bleicher F.', 'Negre D.', 'Cozzone A.J.', 
            'Cortay J.-C.'])

    def test_rx_parser(self):   
        """rx_parser should return expected dict"""
        inputs = [
            ['RX   MEDLINE=22709107; PubMed=12788972; DOI=10.1073/pnas.113'],
            ['RX   PubMed=14577811; '\
                'DOI=10.1597/1545-1569(2003)040<0632:AMMITS>2.0.CO;2;']]
        expects = [
            {'DOI': '10.1073/pnas.113', 'MEDLINE': '22709107', 
                'PubMed': '12788972'},
            {'DOI': '10.1597/1545-1569(2003)040<0632:AMMITS>2.0.CO;2',
                'PubMed': '14577811'}]

        self.assertEqual(map(rx_parser, inputs), expects)

    def test_rc_parser(self):   
        """rc_parser should return expected dict"""
        lines = [
            "RC   PLASMID=R1 (R7268); TRANSPOSON=Tn3;",
            "RC   STRAIN=AL.012, AZ.026;"]

        self.assertEqual(rc_parser(lines),
             {'TRANSPOSON': 'Tn3', 'PLASMID': 'R1 (R7268)', 
             'STRAIN': ['AL.012', 'AZ.026']})

    def test_rt_parser(self):
        """rt_parser should return expected str"""
        lines = [
            'RT   "New insulin-like proteins',
            'RT   analysis and homology modeling.";']
        self.assertEqual( rt_parser(lines),
            'New insulin-like proteins analysis and homology modeling')

    def test_rl_parser(self):
        """rl_parser should return expected str"""
        lines = [
            "RL   J. Mol. Biol. 168:321-331(1983)."]
        self.assertEqual( rl_parser(lines),
            'J. Mol. Biol. 168:321-331(1983)')
    
    def test_rn_parser(self):
        """rn_parser should return expected str"""
        lines = [ "RN   [8]"]
        self.assertEqual( rn_parser(lines), 8)
    
    def test_rg_parser(self):
        """rg_parser should return expected str"""
        lines = [ "RG   The mouse genome sequencing consortium;"]
        self.assertEqual( rg_parser(lines),
            ['The mouse genome sequencing consortium'])

    def test_rp_parser(self):
        """rp_parser should return expected str"""
        lines = [ "RP   X-RAY CRYSTALLOGRAPHY (1.8 ANGSTROMS)."]
        self.assertEqual( rp_parser(lines),
                 'X-RAY CRYSTALLOGRAPHY (1.8 ANGSTROMS)')
        
#################################
# global test data
ft_lines = \
"""FT   CHAIN        29    262       Granzyme A.
FT                                /FTId=PRO_0000027394.
FT   ACT_SITE     69     69       Charge relay system.
FT   VARIANT     121    121       T -> M (in dbSNP:3104233).
FT                                /FTId=VAR_024291.
FT   VARIANT       1      7       unknown  (in a skin tumor).
FT                                /FTId=VAR_005851.
FT   CONFLICT    282    282       R -> Q (in Ref. 18).
FT   STRAND       30     30
FT   STRAND       33     34
FT   TURN         37     38""".splitlines()

ft_expect = \
{'ACT_SITE': [{'Start': 69, 'End': 69, 'Description': 'Charge relay system'}],
 'CHAIN': [{'Description': {'Description': 'Granzyme A',
                            'Id': 'PRO_0000027394'},
            'End': 262,
            'Start': 29}],
 'CONFLICT': [{'Description': {'Comment': 'in Ref. 18',
                               'MutateFrom': 'R',
                               'MutateTo': 'Q'},
               'End': 282,
               'Start': 282}],
 'SecondaryStructure': [('STRAND', 30, 30),
                        ('STRAND', 33, 34),
                        ('TURN', 37, 38)],
 'VARIANT': [{'Description': {'Comment': 'in dbSNP:3104233',
                              'Id': 'VAR_024291',
                              'MutateFrom': 'T',
                              'MutateTo': 'M'},
              'End': 121,
              'Start': 121},
             {'Description': {'Comment': 'in a skin tumor',
                              'Id': 'VAR_005851',
                              'MutateFrom': 'unknown  ',
                              'MutateTo': ''},
              'End': 7,
              'Start': 1}]}

dr_lines =\
"""DR   MIM; 140050; gene.
DR   GO; GO:0001772; C:immunological synapse; TAS.
DR   GO; GO:0005634; C:nucleus; TAS.
DR   GO; GO:0006915; P:apoptosis; TAS.
DR   GO; GO:0006922; P:cleavage of lamin; IDA.
DR   GO; GO:0006955; P:immune response; TAS.
DR   InterPro; IPR001254; Peptidase_S1_S6.
DR   InterPro; IPR001314; Peptidase_S1A.
DR   Pfam; PF00089; Trypsin; 1.""".splitlines()

dr_expect =\
    {'GO': [['GO:0001772', 'C:immunological synapse', 'TAS'],
        ['GO:0005634', 'C:nucleus', 'TAS'],
        ['GO:0006915', 'P:apoptosis', 'TAS'],
        ['GO:0006922', 'P:cleavage of lamin', 'IDA'],
        ['GO:0006955', 'P:immune response', 'TAS']],
    'Pfam': [['PF00089', 'Trypsin', '1']],
    'InterPro': [['IPR001254', 'Peptidase_S1_S6'],
        ['IPR001314', 'Peptidase_S1A']],
    'MIM': [['140050', 'gene']]}

cc_lines = \
"""CC   -!- ALLERGEN: Causes an allergic reaction in human. Binds to IgE.
CC       bovine dander.
CC   -!- ALTERNATIVE PRODUCTS:
CC       Event=Alternative splicing; Named isoforms=3;
CC         Comment=Additional isoforms seem to exist.
CC         confirmation;
CC       Name=1; Synonyms=AIRE-1;
CC         IsoId=O43918-1; Sequence=Displayed;
CC       Name=2; Synonyms=AIRE-2;
CC         IsoId=O43918-2; Sequence=VSP_004089;
CC       Name=3; Synonyms=AIRE-3;
CC         IsoId=O43918-3; Sequence=VSP_004089, VSP_004090;
CC   -!- BIOPHYSICOCHEMICAL PROPERTIES:
CC       Kinetic parameters:
CC         KM=98 uM for ATP;
CC         KM=688 uM for pyridoxal;
CC         Vmax=1.604 mmol/min/mg enzyme;
CC       pH dependence:
CC         Optimum pH is 6.0. Active pH 4.5 to 10.5;
CC   -!- DATABASE: NAME=CD40Lbase;
CC       NOTE=European CD40L defect database (mutation db);
CC       WWW="http://www.expasy.org/cd40lbase/".
CC   -!- DISEASE: Defects in PHKA1 are linked to X-linked muscle
CC       glycogenosis [MIM:311870]. 
CC   -!- DISEASE: Defects in ABCD1 are the cause of recessive X-linked
CC       adrenoleukodystrophy (X-ALD) [MIM:300100]. X-ALD is a rare
CC       phenotype.
CC   -!- INTERACTION:
CC       Self; NbExp=1; IntAct=EBI-123485, EBI-123485;
CC       Q9W158:CG4612; NbExp=1; IntAct=EBI-123485, EBI-89895;
CC       Q9VYI0:fne; NbExp=1; IntAct=EBI-123485, EBI-126770;
CC   -!- MASS SPECTROMETRY: MW=24948; MW_ERR=6; METHOD=MALDI; RANGE=1-228;
CC       NOTE=Ref.2.
CC   --------------------------------------------------------------------------
CC   This SWISS-PROT entry is copyright. It is produced through a collaboration
CC   removed.
CC   --------------------------------------------------------------------------
""".splitlines() 

fake_records_valid = """ID   CYC_BOVIN      STANDARD;      PRT;   104 AA.
AC   ac1; ac2;
DT   dt.
DE   de.
OS   os.
OC   oc.
OX   NCBI_TaxID=9606;
RN   [1]
SQ   SEQUENCE   486 AA;  55639 MW;  D7862E867AD74383 CRC64
     aac cpp
     ggh hh
//
ID   idid  std; prt; 104    #-5
OX   NCBI_TaxID=9606;
DE   dede.
     ggaaccpp
//""".splitlines()


# Run tests if called from the command line
if __name__ == '__main__':
    main()   
